// ignore_for_file: avoid_print, avoid_positional_boolean_parameters

part of kitcod_uikit_flutter;

class KitcodApi {
  KitcodApi._();

  static final KitcodApi _instance = KitcodApi._();

  static KitcodApi get instance => _instance;

/*  final client = Sigv4Client(
    accessKey: ApiConfig.secretKey,
    keyId: ApiConfig.accessKey,
    region: ApiConfig.region,
    serviceName: ApiConfig.service,
  );*/

  Future<AppInfoModel> getAppInfo(final String apiKey) async {
    try {
      final url = Uri.parse(ApiConfig.appInfoPath);
      final response = await http.get(
        url,
        headers: {
          "x-api-key": apiKey,
          "content-type": "application/json",
        },
      );
      if (response.statusCode == 200) {
        final jsonData = jsonDecode(utf8.decode(response.bodyBytes));
        final AppInfoModel appInfoModel =
        AppInfoModel.fromJson(jsonData as Map<String, dynamic>);
        final SharedPreferences prefs = await _prefs;
        await prefs.setString("appInfoData", jsonEncode(jsonData));
        await prefs.setString(
            ApiConfig.kcDefaultCommunityId, appInfoModel.defaultCommunityId!);
        return appInfoModel;
      } else {
        throw Exception("Failed to get App Info");
      }
    } catch (error) {
      rethrow;
    }
  }

  Future<UserProfile> createUser(final String firstName, final String lastName,
      final String profileUrl, final String apiKey) async {
    var newProfileUrl = profileUrl;
    var newFirstName = firstName;
    var newlastName = lastName;
    if (newProfileUrl.isEmpty) {
      newProfileUrl = "https://picsum.photos/200/300";
    }

    if (newFirstName.isEmpty) {
      newFirstName = "Guest";
    }
    if (newlastName.isEmpty) {
      newlastName = "User";
    }
    try {
      final body = {
        "firstName": newFirstName,
        "lastName": newlastName,
        "extProfileImgUrl": newProfileUrl
      };

      final url = Uri.parse(ApiConfig.usersPath);
      final response = await http.post(
        url,
        headers: {
          "x-api-key": apiKey,
          "content-type": "application/json",
        },
        body: jsonEncode(body),
      );
      if (response.statusCode == 201) {
        final jsonData = jsonDecode(utf8.decode(response.bodyBytes));
        final UserProfile userProfile =
        UserProfile.fromJson(jsonData as Map<String, dynamic>);
        return userProfile;
      } else {
        throw Exception("Failed to create post");
      }
    } catch (error) {
      rethrow;
    }
  }

  final Future<SharedPreferences> _prefs = SharedPreferences.getInstance();

  Future<UserProfile> getUserProfileAWSData() async {
    try {
      final SharedPreferences prefs = await _prefs;
      final String? userId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey); // To Do
      // final request = client.request("${ApiConfig.usersPath}$userId");

      /* request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });*/
      // final value = await http.get(request.url, headers: request.headers);
      final url = Uri.parse(ApiConfig.usersPath + userId!);
      final response = await http.get(
        url,
        headers: {
          "x-api-key": apiKey!,
          "content-type": "application/json",
          "kcuserid": userId
        },
      );
      if (response.statusCode == 200) {
        final jsonData = jsonDecode(utf8.decode(response.bodyBytes));
        final UserProfile userProfile =
        UserProfile.fromJson(jsonData as Map<String, dynamic>);

        await prefs.setString("userProfileData", jsonEncode(jsonData));
        return userProfile;
      } else {
        throw Exception("Failed to get User Info");
      }
    } catch (error) {
      final SharedPreferences prefs = await _prefs;
      if (prefs.getString("userProfileData") != null) {
        final String userPref = prefs.getString("userProfileData")!;
        final Map<String, dynamic> userProfileData =
        jsonDecode(userPref) as Map<String, dynamic>;
        final UserProfile userProfile = UserProfile.fromJson(userProfileData);
        return userProfile;
      } else {
        rethrow;
      }
    }
  }

  Future<UserProfile> getUserProfileAWSDataID(final String id) async {
    try {
      final SharedPreferences prefs = await _prefs;
      final String? userId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey); // To Do
      // final request = client.request("${ApiConfig.usersPath}$userId");

      /* request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });*/
      // final value = await http.get(request.url, headers: request.headers);
      final url = Uri.parse(ApiConfig.usersPath + id);
      final response = await http.get(
        url,
        headers: {
          "x-api-key": apiKey!,
          "content-type": "application/json",
          "kcuserid": id
        },
      );
      if (response.statusCode == 200) {
        final jsonData = jsonDecode(utf8.decode(response.bodyBytes));
        final UserProfile userProfile =
        UserProfile.fromJson(jsonData as Map<String, dynamic>);

        await prefs.setString("userProfileData", jsonEncode(jsonData));
        return userProfile;
      } else {
        throw Exception("Failed to get User Info");
      }
    } catch (error) {
      final SharedPreferences prefs = await _prefs;
      if (prefs.getString("userProfileData") != null) {
        final String userPref = prefs.getString("userProfileData")!;
        final Map<String, dynamic> userProfileData =
        jsonDecode(userPref) as Map<String, dynamic>;
        final UserProfile userProfile = UserProfile.fromJson(userProfileData);
        return userProfile;
      } else {
        rethrow;
      }
    }
  }

  Future<UserProfile> getOtherUserProfileID(final String id) async {
    try {
      final SharedPreferences prefs = await _prefs;
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey); // To Do
      // final request = client.request("${ApiConfig.usersPath}$userId");

      /* request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });*/
      // final value = await http.get(request.url, headers: request.headers);
      final url = Uri.parse(ApiConfig.usersPath + id);
      final response = await http.get(
        url,
        headers: {
          "x-api-key": apiKey!,
          "content-type": "application/json",
          "kcuserid": id
        },
      );
      if (response.statusCode == 200) {
        final jsonData = jsonDecode(utf8.decode(response.bodyBytes));
        final UserProfile userProfile =
        UserProfile.fromJson(jsonData as Map<String, dynamic>);

        return userProfile;
      } else {
        throw Exception("Failed to get User Info");
      }
    } catch (error) {
      rethrow;
    }
  }

  Future<CommunitiesList> getGroupDetail(final String id) async {
    try {
      /*final client = Sigv4Client(
        accessKey: ApiConfig.secretKey,
        keyId: ApiConfig.accessKey,
        region: ApiConfig.region,
        serviceName: ApiConfig.service,
      );*/

      // A simple GET-request
      /* final request = client.request("${ApiConfig.communityGetPath}/$id");

      request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });
      final value = await http.get(request.url, headers: request.headers);*/
      final SharedPreferences prefs = await _prefs;
      final String? userId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);

      final url = Uri.parse("${ApiConfig.communityInfoPath}$id");
      final response = await http.get(
        url,
        headers: {
          "x-api-key": apiKey!,
          "content-type": "application/json",
        },
      );
      if (response.statusCode == 200) {
        final jsonData = jsonDecode(utf8.decode(response.bodyBytes));
        final CommunitiesList groupDetails =
        CommunitiesList.fromJson(jsonData as Map<String, dynamic>);
        return groupDetails;
      } else {
        throw Exception("Failed to get Community Id");
      }
    } catch (error) {
      rethrow;
    }
  }

  Future<List<CommunitiesList>> getSuggestedGroupsData(
      final int pageNumber) async {
    try {
      /*final client = Sigv4Client(
        accessKey: ApiConfig.secretKey,
        keyId: ApiConfig.accessKey,
        region: ApiConfig.region,
        serviceName: ApiConfig.service,
      );*/

      const querySize = "5";
      const type = "BUSINESS";
      const longitude = "77.616961";
      const latitude = "12.954044999999999";
      const radius = "25";

      /*final query = {
        "page": pageNumber,
        "size": querySize,
        "longitude": longitude,
        "type": type,
        "latitude": latitude,
        "radius": radius
      };
      final request = client.request(ApiConfig.communityGetPath, query: query);

      request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });
      final value = await http.get(request.url, headers: request.headers);*/

      final SharedPreferences prefs = await _prefs;
      final String? userId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);
      final Dio dio = Dio();
      final value = await dio.get(
        ApiConfig.communityInfoPath,
        options: Options(
          headers: {"x-api-key": apiKey, "content-type": "application/json"},
        ),
        queryParameters: {
          "page": pageNumber,
          "size": querySize,
          "radius": radius
        },
      );
      final jsonData = value.data;

      // final jsonData = jsonDecode(utf8.decode(value.bodyBytes));
      final List<CommunitiesList> feedPostUsers = [];
      // ignore: avoid_dynamic_calls
      for (final feed in jsonData["_embedded"]["communityList"]) {
        // ignore: avoid_dynamic_calls
        final CommunitiesList groupList =
        CommunitiesList.fromJson(feed as Map<String, dynamic>);
        feedPostUsers.add(groupList);
      }
      return feedPostUsers;
    } catch (error) {
      rethrow;
    }
  }

/*
  Future<List<PostList>> getPostAWSData(final String pageNumber) async {
    try {
      const querySize = "25";
      const business = "true";
      final SharedPreferences prefs = await _prefs;
      final String? userId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);
      final query = {
        "page": pageNumber,
        "size": querySize,
        "business": business
      };
      final request = client.request(ApiConfig.postsPath, query: query);

      request.headers.addAll(<String, String>{
        "x-api-key": apiKey!,
        "content-type": "application/json",
        "kcuserid": userId!
      });
      final value = await http.get(request.url, headers: request.headers);

      final jsonData = jsonDecode(utf8.decode(value.bodyBytes));
      final List<PostList> feedPostUsers = [];
      // ignore: avoid_dynamic_calls
      for (final feed in jsonData["_embedded"]["postList"]) {
        // ignore: avoid_dynamic_calls
        final PostList feedPost =
            PostList.fromJson(feed as Map<String, dynamic>);
        feedPostUsers.add(feedPost);
      }
      await prefs.setString("postsData", json.encode(feedPostUsers));

      return feedPostUsers;
    } catch (error) {
      final SharedPreferences prefs = await _prefs;
      if (prefs.getString("postsData") != null) {
        final List<PostList> feedPostUsers =
            json.decode(prefs.getString("postsData")!) as List<PostList>;
        return feedPostUsers;
      } else {
        rethrow;
      }
    }
  }
*/

  Future<List<CommentList>> getPostsCommentsAWSData(final String resourceId,
      final String pageNumber) async {
    try {
      const querySize = "25";
      const business = "true";

      final query = {
        "page": pageNumber,
        "size": querySize,
        "business": business,
        "expandUser": true
      };
      /*   final request = client.request("${ApiConfig.commentsGetPath}/$resourceId",
          query: query);

      request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });
      final value = await http.get(request.url, headers: request.headers);*/

      final SharedPreferences prefs = await _prefs;
      final String? userId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);
      /* final request = client.request("${ApiConfig.kcCommentsGetPath}/$resourceId", query: query);

      request.headers.addAll(<String, String>{
        "x-api-key": apiKey!,
        "content-type": "application/json",
        "kcuserid": userId!
      });
      final value = await http.get(request.url, headers: request.headers);*/
      final Dio dio = Dio();
      final value = await dio.get(
        "${ApiConfig.kcCommentsGetPath}/$resourceId",
        options: Options(
          headers: {
            "x-api-key": apiKey,
            "content-type": "application/json",
            "kcuserid": userId
          },
        ),
        queryParameters: query,
      );

      final jsonData = value.data;
      final List<CommentList> feedPostComments = [];
      // ignore: avoid_dynamic_calls
      if (jsonData["_embedded"] != null) {
        // ignore: avoid_dynamic_calls
        for (final feed in jsonData["_embedded"]["commentList"]) {
          // ignore: avoid_dynamic_calls
          final CommentList feedComments =
          CommentList.fromJson(feed as Map<String, dynamic>);
          feedPostComments.add(feedComments);
        }
      }
      return feedPostComments;
    } catch (error) {
      rethrow;
    }
  }

  Future<List<CommentList>> getPostsChildCommentsAWSData(
      final String resourceId,
      final String pageNumber,
      final String parentId) async {
    try {
      const querySize = "25";
      const business = "false";

      final query = {
        "page": pageNumber,
        "size": querySize,
        "parentId": parentId,
        "expandUser": true
      };
      /* final request = client.request("${ApiConfig.commentsGetPath}/$resourceId",
          query: query);

      request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });
      final value = await http.get(request.url, headers: request.headers);*/

      final SharedPreferences prefs = await _prefs;
      final String? userId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);
      final Dio dio = Dio();
      final value = await dio.get(
        "${ApiConfig.kcCommentsGetPath}/$resourceId",
        options: Options(
          headers: {
            "x-api-key": apiKey,
            "content-type": "application/json",
            "kcuserid": userId
          },
        ),
        queryParameters: {
          "page": pageNumber,
          "size": querySize,
          "parentId": parentId,
          "expandUser": true
        },
      );
      final jsonData = value.data;
      final List<CommentList> feedPostComments = [];
      // ignore: avoid_dynamic_calls
      if (jsonData["_embedded"] != null) {
        // ignore: avoid_dynamic_calls
        for (final feed in jsonData["_embedded"]["commentList"]) {
          // ignore: avoid_dynamic_calls
          final CommentList feedComments =
          CommentList.fromJson(feed as Map<String, dynamic>);
          feedPostComments.add(feedComments);
        }
      }
      return feedPostComments;
    } catch (error) {
      rethrow;
    }
  }

  Future<List<Communities>> getCommunitiesBulkAWSData(
      final List userJoinedCommunities) async {
    try {
      final List body = userJoinedCommunities;

      /* final request = client.request(ApiConfig.communitiesBulkPath,
          method: "POST", body: json.encode(body));

      request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "content-type": "application/json",
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });
      final value = await http.post(request.url,
          headers: request.headers, body: request.body);
*/

      final SharedPreferences prefs = await _prefs;
      final String? kcuserId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);
      final Dio dio = Dio();
      final value = await dio.post(
        ApiConfig.kcCommunitiesBulkPath,
        options: Options(
          headers: {
            "x-api-key": apiKey,
            "content-type": "application/json",
            "kcuserid": kcuserId
          },
        ),
        data: body,
      );

      final jsonData = value.data;
      final List<Communities> bulkGroupResponse = [];
      final List<String> stringData = [];
      // ignore: avoid_dynamic_calls
      for (final feed in jsonData) {
        // ignore: avoid_dynamic_calls
        final Communities feedPost =
        Communities.fromJson(feed as Map<String, dynamic>);
        bulkGroupResponse.add(feedPost);
        stringData.add(jsonEncode(feed));
        final SharedPreferences prefs = await _prefs;
        await prefs.setStringList("communitiesBulkDara", stringData);
      }
      return bulkGroupResponse;
    } catch (error) {
      final SharedPreferences prefs = await _prefs;
      final List<Communities> bulkGroupResponse = [];
      if (prefs.getStringList("communitiesBulkDara") != null) {
        final List<String> postsPref =
        prefs.getStringList("communitiesBulkDara")!;

        // ignore: avoid_dynamic_calls
        for (final community in postsPref) {
          // ignore: avoid_dynamic_calls
          final Communities communityPost = Communities.fromJson(
              jsonDecode(community) as Map<String, dynamic>);
          bulkGroupResponse.add(communityPost);
        }
        return bulkGroupResponse;
      } else {
        rethrow;
      }
    }
  }


  Future<ImageUploadAPIModel?> postImageConfig(
      final List userJoinedCommunities) async {
    try {
      final List body = userJoinedCommunities;


      final SharedPreferences prefs = await _prefs;
      final String? kcuserId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);
      final Dio dio = Dio();
      final value = await dio.post(
        ApiConfig.kcPostConfigUpload,
        options: Options(
          headers: {
            "x-api-key": apiKey,
            "content-type": "application/json",
            "kcuserid": kcuserId
          },
        ),
        data: body,
      );

      if (value.statusCode == 200) {
        final jsonData = value.data;
        final ImageUploadAPIModel membership =
        ImageUploadAPIModel.fromJson(
            jsonData as Map<String, dynamic>);
        return membership;
      } else {
        return null;
      }
    } catch (error) {
     rethrow;
    }
  }


  Future<String?> uploadImage(final String urlString, final CroppedFile imagePath, final String fileName, final String postId) async {
    final Dio dio = Dio();
    final fileBytes = await imagePath.readAsBytes();
    try {

      final response = await dio.put(
          urlString,
          queryParameters: {"x-amz-tagging": "resourceType=POST&resourceId=$postId&displayRank=0"},
          data: base64.encode(fileBytes),
          options: Options(
              headers: {
                "Content-Type": "image/jpg",
              },
              contentType: Headers.formUrlEncodedContentType,
              responseType: ResponseType.plain
          )
      );

      if (response.statusCode != 200) {
        throw Exception("Failed to upload image");
      }else{
        return null;
      }

     /* final formData = FormData.fromMap({
        "file": MultipartFile.fromBytes(fileBytes, filename: fileName),
      });

      final value = await dio.put(urlString, data: formData, options: Options(headers: {"Content-Type": "image/jpg"}));
      if (value.statusCode! >= 200 && value.statusCode! < 400) {
        return value.toString();
      } else {
        return null;
      }*/
    } catch (e) {
      print(e);
    }
  }


  Future<UserCommunityRelationshipList> getCommunityPermission(
      final String userId, final String communityId) async {
    try {
      /*  final query = {
        "userId": userId,
        "communityId": communityId,
      };
      final request = client.request(ApiConfig.getMembershipPath, query: query);

      request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });
      final value = await http.get(request.url, headers: request.headers);
*/

      final SharedPreferences prefs = await _prefs;
      final String? kcuserId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);
      final Dio dio = Dio();
      final value = await dio.get(
        ApiConfig.kcGetMembershipPath,
        options: Options(
          headers: {
            "x-api-key": apiKey,
            "content-type": "application/json",
            "kcuserid": kcuserId
          },
        ),
        queryParameters: {
          "userId": userId,
          "communityId": communityId,
        },
      );

      final jsonData = value.data;
      final UserCommunityRelationshipList membership =
      UserCommunityRelationshipList.fromJson(
          jsonData as Map<String, dynamic>);

      return membership;
    } catch (error) {
      rethrow;
    }
  }

  Future<String?> addPostLikesAws(final String resourceId,
      final String userId) async {
    try {
      final body = {
        "resourceId": resourceId,
        "resourceType": "POST",
        "interactionType": "LIKEOPTION1",
        "userId": userId
      };
      final SharedPreferences prefs = await _prefs;
      final String? kcuserId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);
      final Dio dio = Dio();
      final value = await dio.post(
        ApiConfig.kcInteractionPath,
        options: Options(
          headers: {
            "x-api-key": apiKey,
            "content-type": "application/json",
            "kcuserid": kcuserId
          },
        ),
        data: body,
      );
      /* final request = client.request(ApiConfig.interactionPath,
          method: "POST", body: json.encode(body));

      request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "content-type": "application/json",
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });
      final value = await http.post(request.url,
          headers: request.headers, body: request.body);*/
      if (value.statusCode == 201) {
        return value.toString();
      } else {
        return null;
      }
    } catch (error) {
      rethrow;
    }
  }

  Future<String?> reportPost(final ReportingOption reasons,
      final ReportingOption subReasons,
      final PostList feedl,
      final String textReport) async {
    String reasonId;
    String reasonType;
    if (reasons.textRequired!) {
      reasonId = reasons.id!;
      reasonType = reasons.type!;
    } else {
      reasonId = subReasons.id!;
      reasonType = subReasons.type!;
    }
    try {
      final body = {
        "communityId": feedl.communityId,
        "resourceType": "POST",
        "resourceId": feedl.id,
        "resourceVersion": 1,
        "reasonId": reasonId,
        "reasonType": reasonType,
        "reason": textReport,
      };

      final SharedPreferences prefs = await _prefs;
      final String? kcuserId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);
      final Dio dio = Dio();
      final value = await dio.post(
        ApiConfig.kcReportInteractionPath,
        options: Options(
          headers: {
            "x-api-key": apiKey,
            "content-type": "application/json",
            "kcuserid": kcuserId
          },
        ),
        data: body,
      );

      /* final request = client.request(ApiConfig.reportInteractionPath,
          method: "POST", body: json.encode(body));

      request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "content-type": "application/json",
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });
      final value = await http.post(request.url,
          headers: request.headers, body: request.body);*/
      if (value.statusCode == 201) {
        return value.toString();
      } else {
        return null;
      }
    } catch (error) {
      rethrow;
    }
  }

  Future<String?> joinGroup(final String userId, final String communityId,
      final String invitedBy) async {
    try {
      final body = {
        "userId": userId,
        "communityId": communityId,
        "invitedBy": invitedBy,
      };

      // final request = client.request(ApiConfig.joinMembershipPath,
      //     method: "POST", body: json.encode(body));
      //
      // request.headers.addAll(<String, String>{
      //   "x-api-key": ApiConfig.apiKey,
      //   "content-type": "application/json",
      //   "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      // });
      // final value = await http.post(request.url,
      //     headers: request.headers, body: request.body);
      // print(value);

      final SharedPreferences prefs = await _prefs;
      final String? kcuserId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);
      final Dio dio = Dio();
      final value = await dio.post(
        ApiConfig.kcJoinMembershipPath,
        options: Options(
          headers: {
            "x-api-key": apiKey,
            "content-type": "application/json",
            "kcuserid": kcuserId
          },
        ),
        data: body,
      );

      if (value.statusCode == 201) {
        return value.toString();
      } else {
        return null;
      }
    } catch (error) {
      rethrow;
    }
  }

  Future<String?> addPostOnFeedAws(final String title,
      final String contentHTML,
      final String contentMarkdown,
      final String communityId,
      final String postType,
      final String communityType,
      final List<TextEditingController>? widgetList,
      final DateTime? endingTime,
      final bool isPoll) async {
    try {
      final SharedPreferences prefs = await _prefs;
      final String? userId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);

      final Map<String, dynamic> body;

      if (!isPoll) {
        body = {
          "title": title,
          "contentHTML": contentHTML,
          "contentMarkdown": contentMarkdown,
          "communityId": communityId,
          "postType": "REGULAR" /*,
          "communityType": communityType*/
        };
      } else if (isPoll) {
        int optionCount = 1;
        final optionsArray = [];
        final DateFormat dateFormat =
        DateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
        for (final option in widgetList!) {
          optionsArray
              .add({"type": "POLLOPTION$optionCount", "text": option.text});
          optionCount += 1;
        }
        final Map<String, dynamic> poll = {
          "options": optionsArray,
          "endDateTime": dateFormat.format(endingTime!),
          "timeZoneId": "Asia/Kolkata",
          "timeZoneOffset": "GMT+05:30"
        };
        body = {
          "title": title,
          "communityId": communityId,
          "postType": postType,
          "poll": poll,
          "communityType": "COMMUNITY_CENTER"
        };
      } else {
        body = {};
      }

      /*  final request = client.request(ApiConfig.postsPath,
          method: "POST", body: json.encode(body));

      request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "content-type": "application/json",
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });
      final value = await http.post(request.url,
          headers: request.headers, body: request.body);*/
      final url = Uri.parse(ApiConfig.postsPath);
      final response = await http.post(
        url,
        headers: {
          "x-api-key": apiKey!,
          "content-type": "application/json",
          "kcuserid": userId!
        },
        body: jsonEncode(body),
      );

      if (response.reasonPhrase == "Created") {
        return response.body;
      } else {
        return null;
      }
    } catch (error) {
      rethrow;
    }
  }

  Future<String?> editPostOnFeedAws(final String postId,
      final String title,
      final String contentHTML,
      final String contentMarkdown,
      final String communityId,
      final String postType,
      final String communityType,
      final List<TextEditingController>? widgetList,
      final DateTime? endingTime,
      final bool isPoll,
      final PostList postList) async {
    try {
      final List<int> bytes = utf8.encode(contentMarkdown);

      final Map<String, dynamic> body;
      if (!isPoll) {
        body = {
          "id": postId,
          "title": title,
          "contentHTML": contentHTML,
          "contentMarkdown": utf8.decode(bytes),
          "communityId": communityId,
          "postType": "REGULAR",
          "communityType": communityType,
          "appId": postList.appId,
        };
      } else if (isPoll) {
        int optionCount = 1;
        final optionsArray = [];
        final DateFormat dateFormat =
        DateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
        for (final option in widgetList!) {
          optionsArray
              .add({"type": "POLLOPTION$optionCount", "text": option.text});
          optionCount += 1;
        }
        final Map<String, dynamic> poll = {
          "options": optionsArray,
          "endDateTime": dateFormat.format(endingTime!),
          "timeZoneId": "Asia/Kolkata",
          "timeZoneOffset": "GMT+05:30"
        };
        body = {
          "title": title,
          "communityId": communityId,
          "postType": postType,
          "poll": poll,
          "communityType": "COMMUNITY_CENTER"
        };
      } else {
        body = {};
      }

      /* final request = client.request("${ApiConfig.postsPath}/$postId",
          method: "PUT", body: json.encode(body));

      request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "content-type": "application/json",
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });
      final value = await http.put(request.url,
          headers: request.headers, body: request.body);*/

      final SharedPreferences prefs = await _prefs;
      final String? kcuserId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);
      final Dio dio = Dio();
      final value = await dio.put(
        "${ApiConfig.postsPath}/$postId",
        options: Options(
          headers: {
            "x-api-key": apiKey,
            "content-type": "application/json",
            "kcuserid": kcuserId
          },
        ),
        data: body,
      );

      if (value.statusCode! >= 200 && value.statusCode! < 400) {
        return value.toString();
      } else {
        return null;
      }
    } catch (error) {
      rethrow;
    }
  }

  Future<String?> addPostPollOptionAws(final String resourceId,
      final String userId, final String pollOption) async {
    try {
      final body = {
        "resourceId": resourceId,
        "resourceType": "POST",
        "resourceSubType": "POLL",
        "interactionType": pollOption,
        "userId": userId
      };
/*

      final request = client.request(ApiConfig.interactionPath,
          method: "POST", body: json.encode(body));

      request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "content-type": "application/json",
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });
      final value = await http.post(request.url,
          headers: request.headers, body: request.body);

*/

      final SharedPreferences prefs = await _prefs;
      final String? kcuserId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);
      final Dio dio = Dio();
      final value = await dio.post(
        ApiConfig.kcInteractionPath,
        options: Options(
          headers: {
            "x-api-key": apiKey,
            "content-type": "application/json",
            "kcuserid": kcuserId
          },
        ),
        data: body,
      );

      if (value.statusCode == 201) {
        return value.toString();
      } else {
        return null;
      }
    } catch (error) {
      rethrow;
    }
  }

  Future<String?> addCommentsOnPostAws(final String comment,
      final String resourceId,
      final String userId,
      final String communityId,
      final String? parentId) async {
    try {
      final SharedPreferences prefs = await _prefs;
      final String? userId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);
      var body = {
        "html": comment,
        "userId": userId,
        "communityId": communityId,
        "resourceType": "POST",
        "resourceId": resourceId
      };
      if (parentId != null) {
        body = {
          "html": comment,
          "userId": userId,
          "communityId": communityId,
          "parentId": parentId,
          "resourceType": "POST",
          "resourceId": resourceId
        };
      }
      final url = Uri.parse(ApiConfig.commentsPath);
      final response = await http.post(
        url,
        headers: {
          "x-api-key": apiKey!,
          "content-type": "application/json",
          "kcuserid": userId!
        },
        body: jsonEncode(body),
      );
      /*  final request = client.request(ApiConfig.commentsPath,
          method: "POST", body: json.encode(body));

      request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "content-type": "application/json",
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });
      final value = await http.post(request.url,
          headers: request.headers, body: request.body);*/
      if (response.reasonPhrase == "Created") {
        return response.body;
      } else {
        return null;
      }
    } catch (error) {
      rethrow;
    }
  }

  Future<bool> deleteInteractionAws(final String interactionId) async {
    try {
      /*final request = client.request(
          "${ApiConfig.interactionPath}/$interactionId",
          method: "DELETE");

      request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "content-type": "application/json",
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });*/
      final SharedPreferences prefs = await _prefs;
      final String? kcuserId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);
      final Dio dio = Dio();
      final value =
      await dio.delete("${ApiConfig.kcInteractionPath}/$interactionId",
          options: Options(
            headers: {
              "x-api-key": apiKey,
              "content-type": "application/json",
              "kcuserid": kcuserId
            },
          ));
      /*  final value = await http.delete(request.url,
          headers: request.headers, body: request.body);*/

      if (value.statusCode == 200) {
        return true;
      } else {
        return false;
      }
    } catch (error) {
      rethrow;
    }
  }

  Future<bool> deletePostAws(final String postId) async {
    try {
      /* final request =
          client.request("${ApiConfig.postsPath}/$postId", method: "DELETE");

      request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "content-type": "application/json",
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });*/

      /* final value = await http.delete(request.url,
          headers: request.headers, body: request.body);*/
      final SharedPreferences prefs = await _prefs;
      final String? kcuserId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);
      final Dio dio = Dio();
      final value = await dio.delete("${ApiConfig.postsPath}/$postId",
          options: Options(
            headers: {
              "x-api-key": apiKey,
              "content-type": "application/json",
              "kcuserid": kcuserId
            },
          ));
      if (value.statusCode == 200) {
        return true;
      } else {
        return false;
      }
    } catch (error) {
      rethrow;
    }
  }

  // To do

  // Future<List<CommentList>> getPostCommentData() async {
  //   final response = await http.get(Uri.parse(
  //       "https://628f5fd6dc478523653f60fd.mockapi.io/kitcod-api-comments"));
  //   final jsonData = jsonDecode(utf8.decode(response.bodyBytes));
  //   final List<CommentList> feedPostComments = [];
  //   // ignore: avoid_dynamic_calls
  //   for (final feed in jsonData["_embedded"]["commentList"]) {
  //     // ignore: avoid_dynamic_calls
  //     final CommentList feedComments =
  //         CommentList.fromJson(feed as Map<String, dynamic>);
  //     feedPostComments.add(feedComments);
  //   }
  //   return feedPostComments;
  // }

  Future<List<CommunitiesList>> getGroupsData() async {
    final response = await http.get(Uri.parse(
        "https://628f5fd6dc478523653f60fd.mockapi.io/kitcod-api-communities"));
    final jsonData = jsonDecode(utf8.decode(response.bodyBytes));
    final List<CommunitiesList> feedGroupUsers = [];
    // ignore: avoid_dynamic_calls
    for (final feed in jsonData["_embedded"]["communityList"]) {
      // ignore: avoid_dynamic_calls
      final CommunitiesList feedGroup =
      CommunitiesList.fromJson(feed as Map<String, dynamic>);
      feedGroupUsers.add(feedGroup);
    }
    return feedGroupUsers;
  }

  // Future<List<Communities>> getGroupsBulkData() async {
  //   final response = await http.get(Uri.parse(
  //       "https://testapi.io/api/shahzaibayyub/kitcod-api-bulk-communities"));
  //   final jsonData = jsonDecode(utf8.decode(response.bodyBytes));
  //   final List<Communities> bulkGroupResponse = [];
  //   // ignore: avoid_dynamic_calls
  //   for (final feed in jsonData) {
  //     // ignore: avoid_dynamic_calls
  //     final Communities feedPost =
  //         Communities.fromJson(feed as Map<String, dynamic>);
  //     bulkGroupResponse.add(feedPost);
  //   }
  //   return bulkGroupResponse;
  // }

  // Future<UserProfile> getUserProfileData() async {
  //   final response = await http.get(Uri.parse(
  //       "https://628f5fd6dc478523653f60fd.mockapi.io/kitcodendpoint-user-profile"));
  //   final jsonData = jsonDecode(utf8.decode(response.bodyBytes));
  //   final UserProfile userProfile =
  //       UserProfile.fromJson(jsonData as Map<String, dynamic>);
  //   return userProfile;
  // }

  Future<List<String>> getAdminUserGroupsFromProfileData() async {
    final response = await http.get(Uri.parse(
        "https://628f5fd6dc478523653f60fd.mockapi.io/kitcodendpoint-user-profile"));
    final jsonData = jsonDecode(utf8.decode(response.bodyBytes));
    final UserProfile userProfile =
    UserProfile.fromJson(jsonData as Map<String, dynamic>);
    final List<String> adminCommunities = [];
    for (final UserCommunit community in userProfile.userCommunities!) {
      if (community.userCommunityRoles != null) {
        if (community.userCommunityRoles![0].role == "ADMINISTRATOR") {
          adminCommunities.add(community.communityId!);
        }
      }
    }
    return adminCommunities;
  }

// Future<List<PostList>> getPostsData() async {
//   final response = await http.get(Uri.parse(
//       "https://628f5fd6dc478523653f60fd.mockapi.io/kitcod-api-posts"));
//   final jsonData = jsonDecode(utf8.decode(response.bodyBytes));
//   final List<PostList> feedPostUsers = [];
//   // ignore: avoid_dynamic_calls
//   for (final feed in jsonData["_embedded"]["postList"]) {
//     // ignore: avoid_dynamic_calls
//     final PostList feedPost = PostList.fromJson(feed as Map<String, dynamic>);
//     feedPostUsers.add(feedPost);
//   }
//   return feedPostUsers;
// }

  Future<List<Reporting>> getReportsData() async {
    final response = await http.get(Uri.parse(
        "https://6290aedb27f4ba1c65bf1e4b.mockapi.io/kitcod-api-reporting"));
    final jsonData = jsonDecode(utf8.decode(response.bodyBytes));
    final List<Reporting> feedReportingOptions = [];
    // ignore: avoid_dynamic_calls
    for (final option in jsonData) {
      // ignore: avoid_dynamic_calls
      final Reporting reportingOptions =
      Reporting.fromJson(option as Map<String, dynamic>);
      feedReportingOptions.add(reportingOptions);
    }
    return feedReportingOptions;
  }

  Future<UserCommunityRelationshipList> getGroupMembers(final int pageNumber,
      final String? communityId, final bool isAdmin) async {
    try {
      /* final client = Sigv4Client(
        accessKey: ApiConfig.secretKey,
        keyId: ApiConfig.accessKey,
        region: ApiConfig.region,
        serviceName: ApiConfig.service,
      );*/

      const pageSize = 25;
      var query = {
        "page": pageNumber.toString(),
        "size": pageSize.toString(),
        "communityId": communityId,
        "expandUser": true,
        "fieldNameToInclude": "user.media.proImg",
        "keyMembers": true
      };
      if (isAdmin) {
        query = {
          "page": pageNumber.toString(),
          "size": pageSize.toString(),
          "communityId": communityId,
          "expandUser": true,
          "fieldNameToInclude": "user.media.proImg",
        };
      }

      /*  final url = Uri.parse(ApiConfig.appInfoPath);
      final request = client.request(
          "https://api-sg.collabdeen.com/post-api/posts",
          query: query);

      request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });
      final value = await http.get(request.url, headers: request.headers);*/
      final SharedPreferences prefs = await _prefs;
      final String? userId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);
      // final request = client.request(ApiConfig.postsPath, query: query);

      /*  request.headers.addAll(<String, String>{
        "x-api-key": apiKey!,
        "content-type": "application/json",
        "kcuserid": userId!
      });
      final value = await http.get(request.url, headers: request.headers);*/

      final Dio dio = Dio();
      final value = await dio.get(
        ApiConfig.kcGetMembershipPath,
        options: Options(
          headers: {
            "x-api-key": apiKey,
            "content-type": "application/json",
            "kcuserid": userId
          },
        ),
        queryParameters: query,
      );

      final jsonData = value.data;
      final UserCommunityRelationshipList membership =
      UserCommunityRelationshipList.fromJson(
          jsonData as Map<String, dynamic>);

      return membership;
      // ignore: avoid_dynamic_calls
    } catch (error) {
      rethrow;
    }
  }

  Future<UserCommunityRelationshipList> getGroupMembersRequest(
      final int pageNumber,
      final String? communityId) async {
    try {
      /* final client = Sigv4Client(
        accessKey: ApiConfig.secretKey,
        keyId: ApiConfig.accessKey,
        region: ApiConfig.region,
        serviceName: ApiConfig.service,
      );*/

      const pageSize = 25;
      final query = {
        "page": pageNumber.toString(),
        "size": pageSize.toString(),
        "communityId": communityId,
        "expandUser": true,
        "status": "PENDING_APPROVAL",
        "fieldNameToInclude": "user.media.proImg",
      };

      /*  final url = Uri.parse(ApiConfig.appInfoPath);
      final request = client.request(
          "https://api-sg.collabdeen.com/post-api/posts",
          query: query);

      request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });
      final value = await http.get(request.url, headers: request.headers);*/
      final SharedPreferences prefs = await _prefs;
      final String? userId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);
      // final request = client.request(ApiConfig.postsPath, query: query);

      /*  request.headers.addAll(<String, String>{
        "x-api-key": apiKey!,
        "content-type": "application/json",
        "kcuserid": userId!
      });
      final value = await http.get(request.url, headers: request.headers);*/

      final Dio dio = Dio();
      final value = await dio.get(
        ApiConfig.kcGetMembershipPath,
        options: Options(
          headers: {
            "x-api-key": apiKey,
            "content-type": "application/json",
            "kcuserid": userId
          },
        ),
        queryParameters: query,
      );

      final jsonData = value.data;
      final UserCommunityRelationshipList membership =
      UserCommunityRelationshipList.fromJson(
          jsonData as Map<String, dynamic>);

      return membership;
      // ignore: avoid_dynamic_calls
    } catch (error) {
      rethrow;
    }
  }

  Future<String?> updateRequest(final String? uuid, final UserCommunityRelationshipListElement userCommunityRelationshipListElement) async {
    try {
      final Map<String, dynamic> body = userCommunityRelationshipListElement
          .toJson();
      final SharedPreferences prefs = await _prefs;
      final String? kcuserId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);
      final Dio dio = Dio();
      final value = await dio.put(
        "${ApiConfig.kcGetMembershipPath}/$uuid",
        options: Options(
          headers: {
            "x-api-key": apiKey,
            "content-type": "application/json",
            "kcuserid": kcuserId
          },
        ),
        data: body,
      );

      if (value.statusCode! >= 200 && value.statusCode! < 400) {
        return value.toString();
      } else {
        return null;
      }
    }catch (error) {
      rethrow;
  }
  }
}

