part of kitcod_uikit_flutter;

class BottomSheet extends StatefulWidget {
  const BottomSheet({
    required this.optionsHeading,
    this.selectedOption,
    this.iconColor,
    this.iconBackgroundColor,
    this.headingStyle,
    this.descriptionStyle,
    this.postType,
    this.moreOptions,
    this.context,
    this.imagesSize,
    this.notifyImageList,
    final Key? key,
  }) : super(key: key);

  // -------------------------------- VARIABLES -------------------------------- //
  final Text optionsHeading;
  final Color? iconColor;
  final Color? iconBackgroundColor;
  final TextStyle? headingStyle;
  final TextStyle? descriptionStyle;
  final List<PostType>? postType;
  final List<MoreOptions>? moreOptions;
  final PostType? selectedOption;
  final BuildContext? context;
  final int? imagesSize;
  final Function(List<CroppedFile>? imageList)? notifyImageList;
  @override
  State<BottomSheet> createState() => _BottomSheet();
}

typedef OnPickImageCallback = void Function(
    double? maxWidth, double? maxHeight, int? quality);

class _BottomSheet extends State<BottomSheet> {
  final ImagePicker _picker = ImagePicker();
  final TextEditingController maxWidthController = TextEditingController();
  final TextEditingController maxHeightController = TextEditingController();
  final TextEditingController qualityController = TextEditingController();
  List<CroppedFile>? _imageFileList;
  CroppedFile? _croppedFile;

  void _setImageFileListFromFile(final CroppedFile? value) {
    _imageFileList = value == null ? null : <CroppedFile>[value];
    widget.notifyImageList!(_imageFileList);
  }

  dynamic _pickImageError;

  @override
  void initState() {
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
  }

  Wrap postCreate(final BuildContext context) => Wrap(
        children: [
          Container(
            alignment: Alignment.center,
            child: SizedBox(
              width: MediaQuery.of(context).size.width * 0.15,
              height: 3,
              child: DecoratedBox(
                decoration: BoxDecoration(color: Colors.indigo.shade100),
              ),
            ),
          ),
          Padding(
              padding: const EdgeInsets.all(15), child: widget.optionsHeading),
          if (widget.postType != null)
            for (var option in widget.postType!)
              postCreateOptionSelect(context, option),
          if (widget.moreOptions != null)
            for (var option in widget.moreOptions!)
              postCreateAttachmentSelect(context, option)
        ],
      );

  Row postCreateOptionSelect(
          final BuildContext context, final PostType? selectedOption) =>
      Row(
        children: [
          if (selectedOption == PostType.announcement)
            BottomSheetOption(
              heading: "Announcement",
              description: "Priority notifications to followers",
              icon: KitcodAppIcons.announcement,
              iconColor: widget.iconColor,
              iconBackgroundColor: widget.iconBackgroundColor,
              headingStyle: widget.headingStyle,
              descriptionStyle: widget.descriptionStyle,
            )
          else if (selectedOption == PostType.poll)
            BottomSheetOption(
              heading: "Poll",
              description: "Get opinion from your followers",
              icon: KitcodAppIcons.poll,
              iconColor: widget.iconColor,
              iconBackgroundColor: widget.iconBackgroundColor,
              headingStyle: widget.headingStyle,
              descriptionStyle: widget.descriptionStyle,
            ),
        ],
      );

  Row postCreateAttachmentSelect(
          final BuildContext context, final MoreOptions? selectedOption) =>
      Row(
        children: [
          if (selectedOption == MoreOptions.image || selectedOption == null)
            TextButton(
                style: TextButton.styleFrom(
                    padding: EdgeInsets.zero,
                    minimumSize: const Size(50, 30),
                    tapTargetSize: MaterialTapTargetSize.shrinkWrap,
                    alignment: Alignment.centerLeft),
                onPressed: () {
                  if(widget.imagesSize!<=4) {
                    _onImageButtonPressed(
                        ImageSource.gallery, context: context);
                  }else{
                    showDialog(
                      context: context,
                      builder: (final context) => AlertDialog(
                          title: const Text("Only 5 images can be uploaded at a time"),
                          actions: <Widget>[
                            TextButton(
                              child: const Text("OK"),
                              onPressed: () {
                                Navigator.of(context).pop();
                              },
                            ),
                          ],
                        ),
                    );
                  }
                },
                child: BottomSheetOption(
                  heading: "Image",
                  description: "Add multiple images to your post",
                  icon: KitcodAppIcons.photo,
                  iconColor: widget.iconColor,
                  iconBackgroundColor: widget.iconBackgroundColor,
                  headingStyle: widget.headingStyle,
                  descriptionStyle: widget.descriptionStyle,
                ))
          /*else if (selectedOption == MoreOptions.video)
            TextButton(
              style: TextButton.styleFrom(
                  padding: EdgeInsets.zero,
                  minimumSize: const Size(50, 30),
                  tapTargetSize: MaterialTapTargetSize.shrinkWrap,
                  alignment: Alignment.centerLeft),
              onPressed: () {
                showDialog(
                  context: context,
                  builder: (final _) => AlertDialog(
                    content: TextField(
                      cursorColor: Colors.indigoAccent,
                      onChanged: (final value) {},
                    ),
                    actions: [
                      TextButton(
                        child: const Text("Cancel"),
                        onPressed: () {
                          Navigator.pop(context);
                        },
                      ),
                      TextButton(
                        child: const Text("Post"),
                        onPressed: () {
                          //Navigator.pop(context);
                        },
                      )
                    ],
                  ),
                );
              },
              child: BottomSheetOption(
                heading: "Video",
                description: "Add Video links to your post",
                icon: KitcodAppIcons.video,
                iconColor: widget.iconColor,
                iconBackgroundColor: widget.iconBackgroundColor,
                headingStyle: widget.headingStyle,
                descriptionStyle: widget.descriptionStyle,
              ),
            )*/
          /*else if (selectedOption == MoreOptions.audio)
            BottomSheetOption(
              heading: "Audio",
              description: "Add Audio files to your post",
              icon: KitcodAppIcons.audio,
              iconColor: widget.iconColor,
              iconBackgroundColor: widget.iconBackgroundColor,
              headingStyle: widget.headingStyle,
              descriptionStyle: widget.descriptionStyle,
            )
          else if (selectedOption == MoreOptions.button)
            BottomSheetOption(
              heading: "Button",
              description: "Add a Button to your website or blog",
              icon: KitcodAppIcons.button,
              iconColor: widget.iconColor,
              iconBackgroundColor: widget.iconBackgroundColor,
              headingStyle: widget.headingStyle,
              descriptionStyle: widget.descriptionStyle,
            ),*/
        ],
      );

  Future<void> _onImageButtonPressed(final ImageSource source,
      {final BuildContext? context, final bool isMultiImage = false}) async {
    await _displayPickImageDialog(context!,
        (final maxWidth, final maxHeight, final quality) async {
      try {
        final XFile? pickedFile = await _picker.pickImage(
          source: source,
          maxWidth: maxWidth as double?,
          maxHeight: maxHeight as double?,
          imageQuality: quality as int?,
        );
        setState(() {
          _cropImage(pickedFile!);
          // _setImageFileListFromFile(pickedFile);
        });
      } catch (e) {
        setState(() {
          _pickImageError = e;
        });
      }
    });
  }

  Future<void> _displayPickImageDialog(
          final BuildContext context, final OnPickImageCallback onPick) async =>
      showDialog(
          context: context,
          builder: (final context) => AlertDialog(
                title: const Text("Add optional parameters"),
                content: Column(
                  children: <Widget>[
                    TextField(
                      controller: maxWidthController,
                      keyboardType:
                          const TextInputType.numberWithOptions(decimal: true),
                      decoration: const InputDecoration(
                          hintText: "Enter maxWidth if desired"),
                    ),
                    TextField(
                      controller: maxHeightController,
                      keyboardType:
                          const TextInputType.numberWithOptions(decimal: true),
                      decoration: const InputDecoration(
                          hintText: "Enter maxHeight if desired"),
                    ),
                    TextField(
                      controller: qualityController,
                      keyboardType: TextInputType.number,
                      decoration: const InputDecoration(
                          hintText: "Enter quality if desired"),
                    ),
                  ],
                ),
                actions: <Widget>[
                  TextButton(
                    child: const Text("CANCEL"),
                    onPressed: () {
                      Navigator.of(context).pop();
                    },
                  ),
                  TextButton(
                      child: const Text("PICK"),
                      onPressed: () {
                        final double? width = maxWidthController.text.isNotEmpty
                            ? double.parse(maxWidthController.text)
                            : null;
                        final double? height =
                            maxHeightController.text.isNotEmpty
                                ? double.parse(maxHeightController.text)
                                : null;
                        final int? quality = qualityController.text.isNotEmpty
                            ? int.parse(qualityController.text)
                            : null;
                        onPick(width, height, quality);
                        Navigator.of(context).pop();
                      }),
                ],
              ));


  Future<void> _cropImage(final XFile pickedFile) async {
    if (pickedFile != null) {
      final croppedFile = await ImageCropper().cropImage(
        sourcePath: pickedFile!.path,
        compressQuality: 100,
        uiSettings: [
          AndroidUiSettings(
              toolbarTitle: "Cropper",
              toolbarColor: Colors.deepOrange,
              toolbarWidgetColor: Colors.white,
              initAspectRatio: CropAspectRatioPreset.original,
              lockAspectRatio: false),
          IOSUiSettings(
            title: "Cropper",
          ),
          WebUiSettings(
            context: context,
            boundary: const CroppieBoundary(
              width: 520,
              height: 520,
            ),
            viewPort:
            const CroppieViewPort(width: 480, height: 480, type: "circle"),
            enableExif: true,
            enableZoom: true,
            showZoomer: true,
          ),
        ],
      );
      if (croppedFile != null) {
        setState(() {
          _croppedFile = croppedFile;
          _setImageFileListFromFile(_croppedFile);
        });
      }
    }
  }



// -------------------------------- BUILD FUNCTION -------------------------------- //

  @override
  TextButton build(final BuildContext context) => TextButton(
        onPressed: () => showModalBottomSheet(
            isScrollControlled: true,
            shape: const RoundedRectangleBorder(
                borderRadius: BorderRadius.vertical(
              top: Radius.circular(20),
            )),
            context: context,
            builder: (final context) => Container(
                padding: const EdgeInsets.all(8), child: postCreate(context))),
        child: Container(
          decoration: BoxDecoration(
              border: Border.all(
                color: Colors.grey,
              ),
              borderRadius: const BorderRadius.all(Radius.circular(20))),
          margin: const EdgeInsets.symmetric(horizontal: 10),
          height: MediaQuery.of(context).size.height * 0.09,
          width: MediaQuery.of(context).size.width,
          child: Padding(
            padding: const EdgeInsets.all(8),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: <Widget>[
                Row(children: [
                  if (widget.postType != null)
                    postCreateOptionSelect(context, widget.selectedOption),
                  if (widget.moreOptions != null)
                    BottomSheetOption(
                      heading: "Image",
                      description: "Add multiple images to your post",
                      icon: KitcodAppIcons.photo,
                      iconColor: widget.iconColor,
                      iconBackgroundColor: widget.iconBackgroundColor,
                      headingStyle: widget.headingStyle,
                      descriptionStyle: widget.descriptionStyle,
                    )
                ]),
                const Icon(
                  Icons.keyboard_arrow_down_sharp,
                  //textDirection: TextDirection.RTL!,
                  color: Colors.black,
                )
              ],
            ),
          ),
        ),
      );
}
