part of kitcod_uikit_flutter;

class FeedPostDetails extends StatefulWidget {
  const FeedPostDetails({
    required this.headerView,
    required this.feedPostApi,
    this.reportingOptions,
    this.youtubeUrl,
    this.vimeoUrl,
    this.postedTimeStyle,
    this.feedTitleStyle,
    this.descriptionStyle,
    this.feedAudioTitleStyle,
    this.commentNameStyle,
    this.commentStyle,
    this.commentPostedTimeStyle,
    this.commentSubmitButtonColor,
    this.profileDetails,
    this.isLikedCallback,
    this.noOfLikesCallBack,
    this.notifyParentPostList,
    this.index,
    final Key? key,
  }) : super(key: key);

  // -------------------------------- VARIABLES -------------------------------- //

  final HeaderView headerView;
  final PostList feedPostApi;
  final TextStyle? postedTimeStyle;
  final TextStyle? feedTitleStyle;
  final TextStyle? descriptionStyle;
  final TextStyle? feedAudioTitleStyle;
  final TextStyle? commentNameStyle;
  final TextStyle? commentStyle;
  final TextStyle? commentPostedTimeStyle;
  final Color? commentSubmitButtonColor;
  final UserProfile? profileDetails;
  final String? youtubeUrl;
  final String? vimeoUrl;
  final List<Reporting>? reportingOptions;
  final bool? isLikedCallback;
  final num? noOfLikesCallBack;
  final num? index;
  final Function(bool isliked, num likes, num index)? notifyParentPostList;

  @override
  State<FeedPostDetails> createState() => _FeedPostDetailsState();
}

// -------------------------------- BUILD FUNCTION -------------------------------- //
class _FeedPostDetailsState extends State<FeedPostDetails> {
  final ScrollController _scrollController = ScrollController();
  late List<CommentList> commentsList;
  bool _initialLoading = true;
  String? parentId;
  bool? areLvl2CommentsOpen = false;
  bool? _isLikedPostDetailsCallBack;
  num? _numberOfLikesPostDetails;

  @override
  void initState() {
    loadInitialPostsData();
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
  }

  // ignore: avoid_positional_boolean_parameters
  void updateLikesParentPostDetails(
      final bool isLikedPost, final num likes, final num index) {
    setState(() {
      _isLikedPostDetailsCallBack = isLikedPost;
      _numberOfLikesPostDetails = likes;
      widget.notifyParentPostList!(_isLikedPostDetailsCallBack!,
          _numberOfLikesPostDetails!, widget.index!);
    });
  }

  void refresh(final String? response) {
    if (parentId == null) {
      setState(() {
        final CommentList newComment =
            CommentList.fromJson(jsonDecode(response!) as Map<String, dynamic>);
        newComment.creator = widget.profileDetails;
        commentsList.add(newComment);
      });
    } else {
      setState(() {
        areLvl2CommentsOpen = true;
        parentId = null;
      });
    }
  }

  Future<void> loadInitialPostsData() async {
    commentsList = await KitcodApi.instance
        .getPostsCommentsAWSData(widget.feedPostApi.id!, "0");
    setState(() {
      _initialLoading = false;
    });
  }

  @override
  Widget build(final context) => _initialLoading
      ? Scaffold(
          appBar: widget.headerView,
          body: const Center(
            child: Padding(
              padding: EdgeInsets.all(8),
              child: CircularProgressIndicator(),
            ),
          ),
        )
      : Scaffold(
          appBar: widget.headerView,
          body: SingleChildScrollView(
            controller: _scrollController,
            child: Column(
              children: [
                //groupTagFollow(feedPost, context),
                Padding(
                  padding: const EdgeInsets.only(left: 20, right: 20),
                  child: FeedPostCard(
                    profileDetails: widget.profileDetails,
                    feedPostApi: widget.feedPostApi,
                    reportingOptions: widget.reportingOptions,
                    buildContext: context,
                    descriptionStyle: widget.descriptionStyle,
                    feedAudioTitleStyle: widget.feedAudioTitleStyle,
                    feedTitleStyle: widget.feedTitleStyle,
                    postedTimeStyle: widget.postedTimeStyle,
                    youtubeUrl: widget.youtubeUrl,
                    vimeoUrl: widget.vimeoUrl,
                    isLikedCallback: widget.isLikedCallback,
                    noOfLikesCallBack: widget.noOfLikesCallBack,
                    index: widget.index,
                    notifyParentPostList: updateLikesParentPostDetails,
                  ),
                ),
                ListView.builder(
                  shrinkWrap: true,
                  physics: const NeverScrollableScrollPhysics(),
                  itemCount: commentsList.length,
                  itemBuilder: (final context, final i) => CommentWidget(
                    commentapi: commentsList[i],
                    buildContext: context,
                    commentStyle: widget.commentStyle,
                    nameStyle: widget.commentNameStyle,
                    postedTimeStyle: widget.commentPostedTimeStyle,
                    submitButtonColor: widget.commentSubmitButtonColor,
                    areLvl2CommentsOpen: areLvl2CommentsOpen,
                    func: (final parentIdd) {
                      setState(() {
                        parentId = parentIdd;
                      });
                      _scrollController
                          .jumpTo(_scrollController.position.maxScrollExtent);
                    },
                  ),
                ),

                TextBar(
                    hintText: "Write your comment",
                    resourceId: widget.feedPostApi.id,
                    userId: widget.profileDetails!.id,
                    communityId: widget.feedPostApi.communityId,
                    parentid: parentId,
                    notifyParent: refresh),
              ],
            ),
          ),
        );

// -------------------------------- UI FUNCTION -------------------------------- //
// Widget groupTagFollow(final FeedPost feedPost, final BuildContext context) =>
//     Column(
//       children: [
//         SizedBox(
//           width: MediaQuery.of(context).size.width,
//           height: 3,
//           child: DecoratedBox(
//             decoration: BoxDecoration(color: Colors.indigo.shade50),
//           ),
//         ),
//         Row(
//           mainAxisAlignment: MainAxisAlignment.spaceBetween,
//           children: [
//             Padding(
//               padding: const EdgeInsets.all(10),
//               child: Text(
//                 feedPost.groupTags[0],
//                 style: const TextStyle(fontWeight: FontWeight.bold),
//               ),
//             ),
//             TextButton(
//               onPressed: () {},
//               child: Container(
//                 decoration: BoxDecoration(
//                     border: Border.all(
//                       color: Colors.grey,
//                     ),
//                     borderRadius:
//                         const BorderRadius.all(Radius.circular(20))),
//                 margin: const EdgeInsets.symmetric(horizontal: 10),
//                 height: 35,
//                 width: 73,
//                 alignment: Alignment.center,
//                 child: Row(
//                   children: const [
//                     Icon(
//                       Icons.add,
//                       color: Colors.black,
//                     ),
//                     Text("Follow",
//                         textAlign: TextAlign.center,
//                         style: TextStyle(color: Colors.black)),
//                   ],
//                 ),
//               ),
//             )
//           ],
//         ),
//         SizedBox(
//           width: MediaQuery.of(context).size.width,
//           height: 3,
//           child: DecoratedBox(
//             decoration: BoxDecoration(color: Colors.indigo.shade50),
//           ),
//         ),
//         const SizedBox(height: 10),
//       ],
//     );
}
