part of kitcod_uikit_flutter;

class FeedPostsList extends StatefulWidget {
  const FeedPostsList({
    this.profileDetails,
    this.postedTimeStyle,
    this.feedTitleStyle,
    this.descriptionStyle,
    this.feedAudioTitleStyle,
    this.nameStyle,
    this.userNameStyle,
    this.groupTagsStyle,
    this.commentNameStyle,
    this.commentStyle,
    this.commentPostedTimeStyle,
    this.commentSubmitButtonColor,
    this.communityId,
    final Key? key,
  }) : super(key: key);

  // -------------------------------- VARIABLES -------------------------------- //
  final TextStyle? nameStyle;
  final TextStyle? groupTagsStyle;
  final TextStyle? userNameStyle;
  final TextStyle? postedTimeStyle;
  final TextStyle? feedTitleStyle;
  final TextStyle? descriptionStyle;
  final TextStyle? feedAudioTitleStyle;
  final TextStyle? commentNameStyle;
  final TextStyle? commentStyle;
  final TextStyle? commentPostedTimeStyle;
  final Color? commentSubmitButtonColor;
  final String? communityId;
  final UserProfile? profileDetails;

  @override
  _FeedPostsListState createState() => _FeedPostsListState();
}

// -------------------------------- BUILD FUNCTION -------------------------------- //

class _FeedPostsListState extends State<FeedPostsList> {
  final Future<SharedPreferences> _prefs = SharedPreferences.getInstance();
  final ScrollController _scrollController = ScrollController();
  int pageNumber = 0;
  List<Reporting>? reportingOptions;

  bool _isLoading = false;
  bool _initialLoading = true;
  bool _isLastPage = false;
  List<LikesCallBack> likesCallBackList = [];
  bool? _isLikedCallback;
  num? _numberOfLikesCallback;
  num? _callBackIndex;

  String? youtubeUrl;
  RegExp youtubeExp =
      RegExp(r"^(https?\:\/\/)?((www\.)?youtube\.com|youtu\.be)\/.+$");
  RegExp vimeoExp = RegExp(
      r"(?:http|https)?:?\/?\/?(?:www\.)?(?:player\.)?vimeo\.com\/(?:channels\/(?:\w+\/)?|groups\/(?:[^\/]*)\/videos\/|video\/|)(\d+)(?:|\/\?)");
  List<PostList> feedPostUsers = [];

  Page pageInformation =
      Page(size: 0, totalElements: 0, totalPages: 0, number: 0);
  List<Communities>? userCommunities = [];

  @override
  void initState() {
    getCommunitiesBulk();
    loadInitialPostsData();
    _scrollController.addListener(() async {
      Logger.info(
          "maxScrollExtent: ${_scrollController.position.maxScrollExtent}");
      if (_scrollController.offset >
              _scrollController.position.maxScrollExtent &&
          !_isLastPage) {
        _isLastPage =true;
        setState(() {

          _isLoading = true;
        });
        pageNumber++;
        await getPostAWSDataTest(pageNumber, widget.communityId);
        setState(() {
          _isLoading = false;
        });
      }
    });

    super.initState();
  }

  Future<void> getCommunitiesBulk() async {
    final SharedPreferences prefs = await SharedPreferences.getInstance();
    final List<Communities> bulkGroupResponse = [];
    if (prefs.getStringList("communitiesBulkDara") != null) {
      final List<String> postsPref =
          prefs.getStringList("communitiesBulkDara")!;

      // ignore: avoid_dynamic_calls
      for (final community in postsPref) {
        // ignore: avoid_dynamic_calls
        final Communities communityPost =
            Communities.fromJson(jsonDecode(community) as Map<String, dynamic>);
        userCommunities!.add(communityPost);
      }
    }
  }

  @override
  void dispose() {
    _scrollController.dispose();
    super.dispose();
  }

  void updateLikesParentPostList(
      final bool isLikedPost, final num likes, final num index) {
    setState(() {
      bool present = false;
      for (final LikesCallBack item in likesCallBackList) {
        if (item.index == index) {
          item.isLiked = isLikedPost;
          item.noOfLikes = likes;
          present = true;
        }
      }
      if (!present) {
        likesCallBackList.add(LikesCallBack(
            index: index, isLiked: isLikedPost, noOfLikes: likes));
      }
    });
  }

  bool? checkIsLiked(final List<LikesCallBack> list, final index) {
    for (final LikesCallBack item in list) {
      if (item.index == index) {
        return item.isLiked;
      }
    }
    return null;
  }

  num? checkNumberOfLikes(final List<LikesCallBack> list, final index) {
    for (final LikesCallBack item in list) {
      if (item.index == index) {
        return item.noOfLikes;
      }
    }
    return null;
  }

  Future<void> loadInitialPostsData() async {
    await getPostAWSDataTest(pageNumber, widget.communityId);
    reportingOptions = await KitcodApi.instance.getReportsData();
    setState(() {
      _initialLoading = false;
    });
  }

  Future<void> getPostAWSDataTest(
      final int pageNumber, final String? communityId) async {
    try {
      /* final client = Sigv4Client(
        accessKey: ApiConfig.secretKey,
        keyId: ApiConfig.accessKey,
        region: ApiConfig.region,
        serviceName: ApiConfig.service,
      );*/

      const pageSize = 5;
      const business = "true";
      var query = {
        "page": pageNumber.toString(),
        "size": pageSize.toString(),
        "business": business,
        "expandUser": true
      };
      if (communityId != null) {
        query = {
          "page": pageNumber.toString(),
          "size": pageSize.toString(),
          "communityId": communityId,
          "expandUser": true
        };
      }

      /*  final url = Uri.parse(ApiConfig.appInfoPath);
      final request = client.request(
          "https://api-sg.collabdeen.com/post-api/posts",
          query: query);

      request.headers.addAll(<String, String>{
        "x-api-key": ApiConfig.apiKey,
        "cdauthtoken": "Bearer ${ApiConfig.idToken}"
      });
      final value = await http.get(request.url, headers: request.headers);*/
      final SharedPreferences prefs = await _prefs;
      final String? userId = prefs.getString(ApiConfig.kcUserId); // To Do
      final String? apiKey = prefs.getString(ApiConfig.kcAPIKey);
      // final request = client.request(ApiConfig.postsPath, query: query);

      /*  request.headers.addAll(<String, String>{
        "x-api-key": apiKey!,
        "content-type": "application/json",
        "kcuserid": userId!
      });
      final value = await http.get(request.url, headers: request.headers);*/

      final Dio dio = Dio();
      final value = await dio.get(
        ApiConfig.postsPath,
        options: Options(
          headers: {
            "x-api-key": apiKey,
            "content-type": "application/json",
            "kcuserid": userId
          },
        ),
        queryParameters: query,
      );

      final jsonData = value.data;
      // ignore: avoid_dynamic_calls
      if (jsonData?["_embedded"]?["postList"] != null) {
        final List<String> stringData = [];
        // ignore: avoid_dynamic_calls
        for (final feed in jsonData["_embedded"]["postList"]) {
          final PostList feedPost =
              PostList.fromJson(feed as Map<String, dynamic>);
          for (final community in userCommunities!) {
            // ignore: avoid_dynamic_calls

            if (feedPost.communityId == community.id) {
              feedPost.community = community.body;
            }
          }
          stringData.add(jsonEncode(feed));
          feedPostUsers.add(feedPost);
        }
        // final jsonString = jsonEncode(feedPostUsers);

        pageInformation = Page.fromJson(
            (jsonData as Map<String, dynamic>)["page"] as Map<String, dynamic>);

        _isLastPage = pageInformation.totalPages! <= pageNumber;

        final SharedPreferences prefs = await _prefs;
        await prefs.setStringList("postsData", stringData);
      } else {
        _isLastPage = true;
      }
    } catch (error) {
      _isLastPage = true;
      final SharedPreferences prefs = await _prefs;
      if (prefs.getStringList("postsData") != null) {
        final List<String> postsPref = prefs.getStringList("postsData")!;

        // ignore: avoid_dynamic_calls
        for (final feed in postsPref) {
          // ignore: avoid_dynamic_calls
          final PostList feedPost =
              PostList.fromJson(jsonDecode(feed) as Map<String, dynamic>);
          feedPostUsers.add(feedPost);
        }
      } else {
        rethrow;
      }
    }
  }

  @override
  Widget build(final BuildContext context) => _initialLoading
      ? const Center(
          child: Padding(
          padding: EdgeInsets.all(8),
          child: CircularProgressIndicator(),
        ))
      : feedPostUsers.isEmpty
          ? const Center(
              child: Padding(
                padding: EdgeInsets.all(20),
                child: Text(
                  "No Post Found...",
                  textAlign: TextAlign.center,
                ),
              ),
            )
          :Expanded(child: ListView.builder(
    controller: _scrollController,
    shrinkWrap: true,
    physics: const BouncingScrollPhysics(),
    itemCount: feedPostUsers.length + 1,
    itemBuilder: (final context, final index) =>
    index == feedPostUsers.length
        ? (_isLoading
        ? const Center(
        child: Padding(
          padding: EdgeInsets.all(8),
          child: CircularProgressIndicator(),
        ))
        : const SizedBox.shrink())
        : ListTile(
      title: feedPostCard(
        feedPostUsers[index],
        feedPostUsers[index].contentMarkdown != null &&
            youtubeExp.hasMatch(
                feedPostUsers[index].contentMarkdown!)
            ? youtubeExp.firstMatch(
            feedPostUsers[index].contentMarkdown!)![0]
            : null,
        feedPostUsers[index].contentMarkdown != null &&
            vimeoExp.hasMatch(
                feedPostUsers[index].contentMarkdown!)
            ? vimeoExp.firstMatch(
            feedPostUsers[index].contentMarkdown!)![0]
            : null,
        index,
        context,
      ),
    ),
  ));


  // -------------------------------- UI FUNCTION -------------------------------- //

  TextButton feedPostCard(
          final PostList feedPostApi,
          final String? youtubeUrl,
          final String? vimeoUrl,
          // ignore: avoid_positional_boolean_parameters
          final num index,
          final BuildContext context) =>
      TextButton(
        onPressed: () {
          Navigator.push(
              context,
              MaterialPageRoute(
                builder: (final context) => FeedPostDetails(
                  reportingOptions: reportingOptions,
                  profileDetails: widget.profileDetails,
                  feedPostApi: feedPostApi,
                  youtubeUrl: youtubeUrl,
                  vimeoUrl: vimeoUrl,
                  headerView: HeaderView(
                    backgroundColor: Colors.white.withOpacity(0.1),
                    leftSideIcon: IconButton(
                      onPressed: () {
                        Navigator.pop(context);
                      },
                      icon: const Icon(
                        Icons.arrow_back_outlined,
                        color: Colors.black,
                      ),
                    ),
                    headerAlignment: Alignment.centerLeft,
                  ),
                  descriptionStyle: widget.descriptionStyle,
                  feedAudioTitleStyle: widget.feedAudioTitleStyle,
                  feedTitleStyle: widget.feedTitleStyle,
                  postedTimeStyle: widget.postedTimeStyle,
                  commentNameStyle: widget.commentNameStyle,
                  commentPostedTimeStyle: widget.commentPostedTimeStyle,
                  commentStyle: widget.commentStyle,
                  commentSubmitButtonColor: widget.commentSubmitButtonColor,
                  notifyParentPostList: updateLikesParentPostList,
                  isLikedCallback: checkIsLiked(likesCallBackList, index),
                  noOfLikesCallBack:
                      checkNumberOfLikes(likesCallBackList, index),
                  index: index,
                ),
              ));
        },
        child: FeedPostCard(
            reportingOptions: reportingOptions,
            profileDetails: widget.profileDetails,
            youtubeUrl: youtubeUrl,
            vimeoUrl: vimeoUrl,
            feedPostApi: feedPostApi,
            buildContext: context,
            nameStyle: widget.nameStyle,
            userNameStyle: widget.userNameStyle,
            groupTagsStyle: widget.groupTagsStyle,
            descriptionStyle: widget.descriptionStyle,
            feedAudioTitleStyle: widget.feedAudioTitleStyle,
            feedTitleStyle: widget.feedTitleStyle,
            postedTimeStyle: widget.postedTimeStyle,
            notifyParentPostList: updateLikesParentPostList,
            isLikedCallback: checkIsLiked(likesCallBackList, index),
            noOfLikesCallBack: checkNumberOfLikes(likesCallBackList, index),
            index: index),
      );
}
