part of kitcod_uikit_flutter;

class MemberRequest extends StatefulWidget {
  const MemberRequest(
      {required this.headerView,
      this.communityId,
      this.nameStyle,
      this.userNameStyle,
      final Key? key})
      : super(key: key);
  final String? communityId;
  final TextStyle? nameStyle;
  final TextStyle? userNameStyle;
  final HeaderView headerView;

  @override
  State<MemberRequest> createState() => _MemberRequestState();
}

class _MemberRequestState extends State<MemberRequest> {
  int pageNumber = 0;
  late UserCommunityRelationshipList membership;
  List<UserCommunityRelationshipListElement> userCommunityRelationshipList = [];
  bool _initialLoading = true;
  bool isJoinLoading = false;

  @override
  void initState() {
    loadInitialGroupsMemberData();
    super.initState();
  }

  Future<void> loadInitialGroupsMemberData() async {
    membership = await KitcodApi.instance.getGroupMembersRequest(
      pageNumber,
      widget.communityId,
    );
    setState(() {
      _initialLoading = false;
      if (membership.embedded != null &&
          membership.embedded!.userCommunityRelationshipList != null) {
        userCommunityRelationshipList
            .addAll(membership!.embedded!.userCommunityRelationshipList!!);
      }
    });
  }

  @override
  Widget build(final BuildContext context) => _initialLoading
      ? Scaffold(
          appBar: widget.headerView,
          body: const Center(
              child: Padding(
            padding: EdgeInsets.all(8),
            child: CircularProgressIndicator(),
          )),
        )
      : userCommunityRelationshipList.isEmpty
          ? Scaffold(
              appBar: widget.headerView,
              body: const Center(
                child: Padding(
                  padding: EdgeInsets.all(20),
                  child: Text(
                    "No Requests Found...",
                    textAlign: TextAlign.center,
                  ),
                ),
              ),
            )
          : Scaffold(
              appBar: widget.headerView,
              body: ListView.builder(
                  itemCount: userCommunityRelationshipList.length,
                  itemBuilder: (final context, final index) => Column(
                        children: [
                          ListTile(
                            leading: Container(
                              width: 50,
                              height: 50,
                              decoration: BoxDecoration(
                                shape: BoxShape.circle,
                                image: DecorationImage(
                                  fit: BoxFit.fill,
                                  image: NetworkImage(
                                    userCommunityRelationshipList[index]!
                                            .user
                                            ?.extProfileImgUrl ??
                                        "https://picsum.photos/200",
                                  ),
                                ),
                              ),
                            ),
                            title: Text(
                                "${userCommunityRelationshipList[index]!.user?.firstName} ${userCommunityRelationshipList[index]!.user?.lastName}"),
                          ),
                          Row(
                            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                            children: [
                              ElevatedButton(
                                onPressed: () {
                                  // Accept button press
                                  isJoinLoading = true;
                                  acceptGroup(
                                      userCommunityRelationshipList[index]!,
                                      "APPROVED",
                                      index);
                                },
                                style: ElevatedButton.styleFrom(
                                  primary: Colors.green,
                                ),
                                child: isJoinLoading
                                    ? const CircularProgressIndicator()
                                    : Row(
                                        mainAxisSize: MainAxisSize.min,
                                        children: const [
                                          Text("Accept"),
                                          SizedBox(width: 10),
                                          Icon(Icons.approval)
                                        ],
                                      ),
                              ),
                              ElevatedButton(
                                onPressed: () {
                                  // Reject button press
                                },
                                style: ElevatedButton.styleFrom(
                                  primary: Colors.red,
                                ),
                                child: Row(
                                  mainAxisSize: MainAxisSize.min,
                                  children: const [
                                    Text("Reject"),
                                    SizedBox(width: 10),
                                    Icon(Icons.cancel_outlined)
                                  ],
                                ),
                              ),
                            ],
                          ),
                          const Divider(),
                        ],
                      )),
            );

  Future<void> acceptGroup(
      final UserCommunityRelationshipListElement
          userCommunityRelationshipListElement,
      final String status,
      final int index) async {
    userCommunityRelationshipListElement.status = status;
    userCommunityRelationshipListElement.user = null;
    await KitcodApi.instance.updateRequest(
        userCommunityRelationshipListElement?.uuid,
        userCommunityRelationshipListElement);
    setState(() {
      isJoinLoading = false;
      userCommunityRelationshipList.removeAt(index);
    });
  }
}
