part of kitcod_uikit_flutter;

class GroupMember extends StatefulWidget {
  const GroupMember(
      {this.communityId,
      this.nameStyle,
      this.userNameStyle,
      this.isAdmin,
      this.groupStatus,
      final Key? key})
      : super(key: key);
  final String? communityId;
  final TextStyle? nameStyle;
  final TextStyle? userNameStyle;
  final bool? isAdmin;
  final String? groupStatus;

  @override
  State<GroupMember> createState() => _GroupMemberState();
}

class _GroupMemberState extends State<GroupMember>
    with AutomaticKeepAliveClientMixin {
  int pageNumber = 0;
  late UserCommunityRelationshipList membership;
  List<UserCommunityRelationshipListElement> userCommunityRelationshipList = [];
  bool _initialLoading = true;

  @override
  bool get wantKeepAlive => true;

  @override
  void initState() {
    loadInitialGroupsMemberData();
    super.initState();
  }

  Future<void> loadInitialGroupsMemberData() async {
    userCommunityRelationshipList.clear();
    membership = await KitcodApi.instance.getGroupMembers(
      pageNumber,
      widget.communityId,
      widget.isAdmin!,
    );
    setState(() {
      _initialLoading = false;
      if (membership.embedded != null &&
          membership.embedded!.userCommunityRelationshipList != null) {
        userCommunityRelationshipList
            .addAll(membership!.embedded!.userCommunityRelationshipList!!);
      }
    });
  }

  @override
  Widget build(final BuildContext context) => _initialLoading
      ? const Center(
          child: Padding(
          padding: EdgeInsets.all(8),
          child: CircularProgressIndicator(),
        ))
      : userCommunityRelationshipList.isEmpty
          ? const Center(
              child: Padding(
                padding: EdgeInsets.all(20),
                child: Text(
                  "No Members Found...",
                  textAlign: TextAlign.center,
                ),
              ),
            )
          : Column(
              children: [
                if (widget.isAdmin! && widget.groupStatus == "PRIVATE")
                  DecoratedBox(
                    decoration: BoxDecoration(
                      color: Colors.grey[200],
                      borderRadius: const BorderRadius.all(Radius.circular(8)),
                    ),
                    child: Padding(
                      padding: const EdgeInsets.all(8),
                      child: GestureDetector(
                        onTap: () {
                          Navigator.push(
                              context,
                              MaterialPageRoute(
                                  builder: (final context) =>
                                      MemberRequest(
                                        communityId: widget.communityId,
                                        nameStyle: widget.nameStyle,
                                        userNameStyle: widget.userNameStyle,
                                        headerView: HeaderView(
                                          backgroundColor: Colors.white.withOpacity(0.1),
                                          leftSideIcon: IconButton(
                                            onPressed: () {
                                              Navigator.pop(context);
                                            },
                                            icon: const Icon(
                                              Icons.arrow_back_outlined,
                                              color: Colors.black,
                                            ),
                                          ),
                                          header: const Text(
                                            "Requests",
                                            style: TextStyle(color: Colors.black),
                                          ),
                                          headerAlignment: Alignment.centerLeft,
                                        ),
                                      )
                              )
                          );
                        },
                        child: Row(
                          children: const [
                            Expanded(child: Text("Requests")),
                            Icon(Icons.arrow_forward),
                          ],
                        ),
                      )


                    ),

                  ),
                Expanded(
                  child: RefreshIndicator(onRefresh: loadInitialGroupsMemberData,
                  child: ListView.builder(
                    itemCount: userCommunityRelationshipList.length,
                    itemBuilder: (final context, final index) => ListTile(
                        title: ProfileDetails(
                            name:
                            "${userCommunityRelationshipList[index]!.user?.firstName} ${userCommunityRelationshipList[index]!.user?.lastName}",
                            profileImageUrl:
                            userCommunityRelationshipList[index]!
                                .user
                                ?.extProfileImgUrl ??
                                "https://picsum.photos/200",
                            nameStyle: widget.nameStyle,
                            userNameStyle: widget.userNameStyle,
                            profileDetails:
                            userCommunityRelationshipList[index]!.user,
                            isMemberScreen: true,
                            userCommunityRoles:
                            userCommunityRelationshipList[index]!
                                .userCommunityRoles)),
                  ),)


                ),
              ],
            );

/*Column(children: [
    const Padding(padding: EdgeInsets.all(8),
    child:  Text("Requests")),
    Expanded(child: ListView.builder(
      itemCount: userCommunityRelationshipList.length,
      itemBuilder: (final context, final index) => ListTile(
          title: ProfileDetails(
              name:
              "${userCommunityRelationshipList[index]!.user?.firstName} ${userCommunityRelationshipList[index]!.user?.lastName}",
              profileImageUrl: userCommunityRelationshipList[index]!
                  .user
                  ?.extProfileImgUrl ??
                  "https://picsum.photos/200",
              nameStyle: widget.nameStyle,
              userNameStyle: widget.userNameStyle,
              profileDetails:
              userCommunityRelationshipList[index]!.user,
              isMemberScreen: true,userCommunityRoles: userCommunityRelationshipList[index]!.userCommunityRoles)),
    ))
  ],);*/

}
