part of kitcod_uikit_flutter;

class GroupPage extends StatefulWidget {
  const GroupPage({
    required this.groupImageUrl,
    required this.groupTitle,
    required this.members,
    required this.groupPageStatus,
    required this.groupPageAbout,
    this.membersListUrl,
    this.groupPageTitleStyle,
    this.groupPageMembersStyle,
    this.groupPageStatusStyle,
    this.groupPageAboutStyle,
    this.isJoined,
    this.nameStyle,
    this.userNameStyle,
    this.groupTagsStyle,
    this.iconBackgroundColor,
    this.headingStyle,
    this.descriptionStyle,
    this.postedTimeStyle,
    this.feedTitleStyle,
    this.feedDescriptionStyle,
    this.feedAudioTitleStyle,
    this.commentNameStyle,
    this.commentStyle,
    this.commentPostedTimeStyle,
    this.commentSubmitButtonColor,
    this.profileDetailsResponse,
    this.communityId,
    this.communityCreatedBy,
    final Key? key,
  }) : super(key: key);

  final String groupImageUrl;
  final String groupTitle;
  final String members;
  final String? communityId;
  final String? communityCreatedBy;
  final List<String>? membersListUrl;
  final String groupPageStatus;
  final String groupPageAbout;
  final bool? isJoined;
  final TextStyle? groupPageTitleStyle;
  final TextStyle? groupPageMembersStyle;
  final TextStyle? groupPageStatusStyle;
  final TextStyle? groupPageAboutStyle;
  final TextStyle? nameStyle;
  final TextStyle? groupTagsStyle;
  final TextStyle? userNameStyle;
  final TextStyle? headingStyle;
  final TextStyle? descriptionStyle;
  final Color? iconBackgroundColor;
  final TextStyle? postedTimeStyle;
  final TextStyle? feedTitleStyle;
  final TextStyle? feedDescriptionStyle;
  final TextStyle? feedAudioTitleStyle;
  final TextStyle? commentNameStyle;
  final TextStyle? commentStyle;
  final TextStyle? commentPostedTimeStyle;
  final Color? commentSubmitButtonColor;
  final UserProfile? profileDetailsResponse;

  @override
  State<GroupPage> createState() => _GroupPageState();
}

class _GroupPageState extends State<GroupPage>
    with SingleTickerProviderStateMixin {
  String? buttonText;
  late UserCommunityRelationshipList membership;
  bool initialLoading = true;
  bool isJoinLoading = false;
  late final _tabController = TabController(vsync: this, length: 2);
  bool isAdmin = false;

  @override
  void initState() {
    checkMembership();
    super.initState();
  }

  @override
  void dispose() {
    _tabController.dispose();
    super.dispose();
  }

  Future<void> checkMembership() async {
    membership = await KitcodApi.instance.getCommunityPermission(
      widget.profileDetailsResponse!.id!,
      widget.communityId!,
    );

    if (membership.embedded == null && widget.groupPageStatus == "PUBLIC") {
      setState(() {
        buttonText = "Follow";
      });
    } else if (membership.embedded == null &&
        widget.groupPageStatus == "PRIVATE") {
      setState(() {
        buttonText = "Join";
      });
    } else if (membership
            .embedded!.userCommunityRelationshipList!.first.status ==
        "FOLLOWING") {
      setState(() {
        buttonText = "Invite";
      });
    } else if (membership
            .embedded!.userCommunityRelationshipList!.first.status ==
        "PENDING") {
      setState(() {
        buttonText = "Pending";
      });
    }else if (membership
        .embedded!.userCommunityRelationshipList!.first.status ==
        "APPROVED") {
      setState(() {
        buttonText = "Invite";
      });
    }


    setState(() {
      if (membership.embedded != null) {
        if (membership!.embedded!.userCommunityRelationshipList!.first
            .userCommunityRoles!.isNotEmpty) {
          for (final r in membership!.embedded!.userCommunityRelationshipList!
              .first.userCommunityRoles!!) {
            if (r.role == "ADMINISTRATOR") {
              isAdmin = true;
              break;
            }
          }
        }
      }

      initialLoading = false;

    });
  }

  Future<void> joinGroup() async {
    await KitcodApi.instance.joinGroup(widget.profileDetailsResponse!.id!,
        widget.communityId!, widget.communityCreatedBy!);
    if (widget.groupPageStatus == "PUBLIC") {
      setState(() {
        isJoinLoading = false;
        buttonText = "Invite";
      });
    } else if (widget.groupPageStatus == "PRIVATE") {
      setState(() {
        isJoinLoading = false;
        buttonText = "Pending";
      });
    }
  }

  @override
  Scaffold build(final context) => initialLoading
      ? const Scaffold(
          body: Center(
            child: Padding(
              padding: EdgeInsets.all(8),
              child: CircularProgressIndicator(),
            ),
          ),
        )
      : Scaffold(
          backgroundColor: Colors.white,
          appBar: AppBar(
              bottom: TabBar(
                labelColor: Colors.white,
                unselectedLabelColor: Colors.grey,
                controller: _tabController,
                tabs: const <Widget>[
                  Tab(icon: Icon(Icons.home)),
                  Tab(icon: Icon(Icons.group)),
                ],
              ),
              title: const Text("Group Details")),
          body: TabBarView(
            controller: _tabController,
            children: [
              SingleChildScrollView(
                child: SafeArea(
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Stack(children: <Widget>[
                        CachedNetworkImage(
                          imageUrl: widget.groupImageUrl,
                          width: MediaQuery.of(context).size.width,
                          height: MediaQuery.of(context).size.height * 0.3,
                          fit: BoxFit.cover,
                          placeholder: (final context, final url) =>
                              const CircularProgressIndicator(),
                          errorWidget:
                              (final context, final url, final error) =>
                                  const Icon(Icons.error),
                        ),
                        Positioned(
                            top: 10,
                            child: TextButton(
                                onPressed: () => {Navigator.pop(context)},
                                style: ElevatedButton.styleFrom(
                                    shape: const CircleBorder(),
                                    padding: const EdgeInsets.all(10),
                                    onPrimary: Colors.black,
                                    primary: Colors.white.withOpacity(0.7)),
                                child: const Icon(
                                    Icons.arrow_back_ios_new_sharp))),
                        Positioned(
                            right: 10,
                            top: 10,
                            child: TextButton(
                                onPressed: () => {},
                                style: ElevatedButton.styleFrom(
                                    shape: const CircleBorder(),
                                    padding: const EdgeInsets.all(10),
                                    onPrimary: Colors.black,
                                    primary: Colors.white.withOpacity(0.7)),
                                child: const Icon(Icons.menu_outlined))),
                      ]),
                      Padding(
                        padding:
                            const EdgeInsets.only(left: 20, top: 20, right: 20),
                        child: Text(
                          widget.groupTitle,
                          style: widget.groupPageTitleStyle ??
                              const TextStyle(
                                fontSize: 30,
                                fontWeight: FontWeight.bold,
                                color: Colors.black,
                              ),
                        ),
                      ),
                      Padding(
                        padding: const EdgeInsets.only(left: 20, top: 10),
                        child: RichText(
                          text: TextSpan(
                              text: widget.groupPageStatus,
                              style: widget.groupPageStatusStyle ??
                                  const TextStyle(
                                      fontSize: 17, color: Colors.grey),
                              children: <TextSpan>[
                                if (widget.members != "")
                                  TextSpan(
                                    text: " - ${widget.members}",
                                    style: widget.groupPageMembersStyle ??
                                        const TextStyle(
                                            fontSize: 17,
                                            fontWeight: FontWeight.bold,
                                            color: Colors.black),
                                  )
                              ]),
                        ),
                      ),
                      if (!widget.isJoined!)
                        isJoinLoading
                            ? const Center(
                                child: Padding(
                                padding: EdgeInsets.all(8),
                                child: CircularProgressIndicator(),
                              ))
                            : Padding(
                                padding: const EdgeInsets.only(
                                    left: 20, top: 20, right: 20),
                                child: Button(
                                  name: buttonText!,
                                  onPressed: () {
                                    setState(
                                      () {
                                        if (buttonText == "Follow" ||
                                            buttonText == "Join") {
                                          isJoinLoading = true;
                                          joinGroup();
                                        }
                                      },
                                    );
                                  },
                                  backgroundColor: Colors.indigo,
                                  textStyle:
                                      const TextStyle(color: Colors.white),
                                  height: 40,
                                  width: MediaQuery.of(context).size.width,
                                ),
                              ),
                      if (widget.isJoined!)
                        Padding(
                          padding: const EdgeInsets.only(
                              left: 20, top: 20, right: 20),
                          child: Button(
                            name: "Invite",
                            onPressed: () {},
                            backgroundColor: Colors.indigo,
                            textStyle: const TextStyle(color: Colors.white),
                            height: 40,
                            width: MediaQuery.of(context).size.width,
                          ),
                          // child: Row(
                          //   children: [
                          // RowSuper(
                          //   innerDistance: -10,
                          //   children: [
                          //     if (widget.membersListUrl != null)
                          //       for (String member in widget.membersListUrl!)
                          //         CircleAvatar(
                          //           radius: 15,
                          //           backgroundImage: NetworkImage(member),
                          //         ),
                          //   ],
                          // ),

                          //],
                        ),
                      if (widget.groupPageAbout != "")
                        const Padding(
                          padding:
                              EdgeInsets.only(left: 20, top: 20, right: 20),
                          child: Text(
                            "About",
                            style: TextStyle(
                              fontSize: 20,
                              fontWeight: FontWeight.bold,
                              color: Colors.black,
                            ),
                          ),
                        ),
                      if (widget.groupPageAbout != "")
                        Padding(
                          padding: const EdgeInsets.only(
                              left: 20, top: 10, right: 20, bottom: 20),
                          child: Text(widget.groupPageAbout,
                              style: widget.groupPageAboutStyle ??
                                  const TextStyle(
                                      fontSize: 15, color: Colors.grey)),
                        ),
                      FeedPostsList(
                        profileDetails: widget.profileDetailsResponse,
                        nameStyle: widget.nameStyle,
                        userNameStyle: widget.userNameStyle,
                        groupTagsStyle: widget.groupTagsStyle,
                        descriptionStyle: widget.feedDescriptionStyle,
                        feedAudioTitleStyle: widget.feedAudioTitleStyle,
                        feedTitleStyle: widget.feedTitleStyle,
                        postedTimeStyle: widget.postedTimeStyle,
                        commentNameStyle: widget.commentNameStyle,
                        commentPostedTimeStyle: widget.commentPostedTimeStyle,
                        commentStyle: widget.commentStyle,
                        commentSubmitButtonColor:
                            widget.commentSubmitButtonColor,
                        communityId: widget.communityId,
                      ),
                    ],
                  ),
                ),
              ),
              GroupMember(
                communityId: widget.communityId,
                nameStyle: widget.nameStyle,
                userNameStyle: widget.userNameStyle,
                isAdmin: isAdmin,
                groupStatus: widget.groupPageStatus,
              )
            ],
          ),
          /*body: SingleChildScrollView(
            child: SafeArea(
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Stack(children: <Widget>[
                    CachedNetworkImage(
                      imageUrl: widget.groupImageUrl,
                      width: MediaQuery.of(context).size.width,
                      height: MediaQuery.of(context).size.height * 0.3,
                      fit: BoxFit.cover,
                      placeholder: (final context, final url) =>
                          const CircularProgressIndicator(),
                      errorWidget: (final context, final url, final error) =>
                          const Icon(Icons.error),
                    ),
                    Positioned(
                        top: 10,
                        child: TextButton(
                            onPressed: () => {Navigator.pop(context)},
                            style: ElevatedButton.styleFrom(
                                shape: const CircleBorder(),
                                padding: const EdgeInsets.all(10),
                                onPrimary: Colors.black,
                                primary: Colors.white.withOpacity(0.7)),
                            child: const Icon(Icons.arrow_back_ios_new_sharp))),
                    Positioned(
                        right: 10,
                        top: 10,
                        child: TextButton(
                            onPressed: () => {},
                            style: ElevatedButton.styleFrom(
                                shape: const CircleBorder(),
                                padding: const EdgeInsets.all(10),
                                onPrimary: Colors.black,
                                primary: Colors.white.withOpacity(0.7)),
                            child: const Icon(Icons.menu_outlined))),
                  ]),
                  Padding(
                    padding:
                        const EdgeInsets.only(left: 20, top: 20, right: 20),
                    child: Text(
                      widget.groupTitle,
                      style: widget.groupPageTitleStyle ??
                          const TextStyle(
                            fontSize: 30,
                            fontWeight: FontWeight.bold,
                            color: Colors.black,
                          ),
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.only(left: 20, top: 10),
                    child: RichText(
                      text: TextSpan(
                          text: widget.groupPageStatus,
                          style: widget.groupPageStatusStyle ??
                              const TextStyle(fontSize: 17, color: Colors.grey),
                          children: <TextSpan>[
                            if (widget.members != "")
                              TextSpan(
                                text: " - ${widget.members}",
                                style: widget.groupPageMembersStyle ??
                                    const TextStyle(
                                        fontSize: 17,
                                        fontWeight: FontWeight.bold,
                                        color: Colors.black),
                              )
                          ]),
                    ),
                  ),
                  if (!widget.isJoined!)
                    isJoinLoading
                        ? const Center(
                            child: Padding(
                            padding: EdgeInsets.all(8),
                            child: CircularProgressIndicator(),
                          ))
                        : Padding(
                            padding: const EdgeInsets.only(
                                left: 20, top: 20, right: 20),
                            child: Button(
                              name: buttonText!,
                              onPressed: () {
                                setState(
                                  () {

                                    if (buttonText == "Follow" ||
                                        buttonText == "Join") {
                                      isJoinLoading = true;
                                      joinGroup();
                                    }
                                  },
                                );
                              },
                              backgroundColor: Colors.indigo,
                              textStyle: const TextStyle(color: Colors.white),
                              height: 40,
                              width: MediaQuery.of(context).size.width,
                            ),
                          ),
                  if (widget.isJoined!)
                    Padding(
                      padding:
                          const EdgeInsets.only(left: 20, top: 20, right: 20),
                      child: Button(
                        name: "Invite",
                        onPressed: () {},
                        backgroundColor: Colors.indigo,
                        textStyle: const TextStyle(color: Colors.white),
                        height: 40,
                        width: MediaQuery.of(context).size.width,
                      ),
                      // child: Row(
                      //   children: [
                      // RowSuper(
                      //   innerDistance: -10,
                      //   children: [
                      //     if (widget.membersListUrl != null)
                      //       for (String member in widget.membersListUrl!)
                      //         CircleAvatar(
                      //           radius: 15,
                      //           backgroundImage: NetworkImage(member),
                      //         ),
                      //   ],
                      // ),

                      //],
                    ),
                  if (widget.groupPageAbout != "")
                    const Padding(
                      padding: EdgeInsets.only(left: 20, top: 20, right: 20),
                      child: Text(
                        "About",
                        style: TextStyle(
                          fontSize: 20,
                          fontWeight: FontWeight.bold,
                          color: Colors.black,
                        ),
                      ),
                    ),
                  if (widget.groupPageAbout != "")
                    Padding(
                      padding: const EdgeInsets.only(
                          left: 20, top: 10, right: 20, bottom: 20),
                      child: Text(widget.groupPageAbout,
                          style: widget.groupPageAboutStyle ??
                              const TextStyle(
                                  fontSize: 15, color: Colors.grey)),
                    ),
                  FeedPostsList(
                    profileDetails: widget.profileDetailsResponse,
                    nameStyle: widget.nameStyle,
                    userNameStyle: widget.userNameStyle,
                    groupTagsStyle: widget.groupTagsStyle,
                    descriptionStyle: widget.feedDescriptionStyle,
                    feedAudioTitleStyle: widget.feedAudioTitleStyle,
                    feedTitleStyle: widget.feedTitleStyle,
                    postedTimeStyle: widget.postedTimeStyle,
                    commentNameStyle: widget.commentNameStyle,
                    commentPostedTimeStyle: widget.commentPostedTimeStyle,
                    commentStyle: widget.commentStyle,
                    commentSubmitButtonColor: widget.commentSubmitButtonColor,
                    communityId: widget.communityId,
                  ),
                ],
              ),
            ),
          ),*/
        );
}
