// ignore_for_file: prefer_if_null_operators

part of kitcod_uikit_flutter;

class ProfileDetails extends StatefulWidget {
  const ProfileDetails({
    this.reportingOptions,
    this.name,
    this.feedPostApi,
    this.profileImageUrl,
    this.nameStyle,
    this.userNameStyle,
    this.groupTagsStyle,
    this.postedTo,
    this.profileDetails,
    this.groupDetailUpdated,
    this.isCommentScreen,
    this.isMemberScreen,
    this.userCommunityRoles,
    final Key? key,
  }) : super(key: key);

  // -------------------------------- VARIABLES -------------------------------- //

  final String? name;
  final String? profileImageUrl;
  final PostList? feedPostApi;
  final String? postedTo;
  final TextStyle? nameStyle;
  final TextStyle? groupTagsStyle;
  final TextStyle? userNameStyle;
  final UserProfile? profileDetails;
  final List<Reporting>? reportingOptions;
  final CommunitiesList? groupDetailUpdated;
  final bool? isCommentScreen;
  final bool? isMemberScreen;
  final List<UserCommunityRole>? userCommunityRoles;

  @override
  State<ProfileDetails> createState() => _ProfileDetails();
}

// -------------------------------- BUILD FUNCTION -------------------------------- //
class _ProfileDetails extends State<ProfileDetails> {
  late PageController pageController;
  CommunitiesList? groupDetail;
  bool _initialLoading = true;
  bool _isReportLoading = false;
  int optionIndex = 0;
  int subReasonIndex = 0;
  String? _userId = "";
  final TextEditingController reportTextController = TextEditingController();
  final GlobalKey<ScaffoldState> _modelScaffoldKey = GlobalKey<ScaffoldState>();
  String role = "Member";
  final _random = Random();
  Color _randomColor = Colors.black;
  @override
  void initState() {
    pageController = PageController();
    _randomColor = _randomColorGenerator();
    if (widget.userCommunityRoles != null) {
      for (final r in widget.userCommunityRoles!!) {
        if (r.role == "ADMINISTRATOR") {
          role = "Administrator";
          break;
        }
      }
    }

    setState(() {
      role = role;
      groupDetail = widget.groupDetailUpdated;
    });

    getSharedPrefs();
    loadInitialGroupDetail();
    super.initState();
  }
  /*Color _randomColorGenerator() {
    final colors = [Colors.red, Colors.green, Colors.blue, Colors.purple];
    return colors[_random.nextInt(colors.length)];
  }*/
  Color _randomColorGenerator() => Color.fromARGB(
    _random.nextInt(255),
    _random.nextInt(255),
    _random.nextInt(255),
    _random.nextInt(255),
  );
  Future<void> getSharedPrefs() async {
    final SharedPreferences prefs = await SharedPreferences.getInstance();
    _userId = prefs.getString(ApiConfig.kcUserId);
  }

  Future<void> loadInitialGroupDetail() async {
    if (widget.feedPostApi != null) {
      if (widget.feedPostApi!.community != null) {
        groupDetail = widget.feedPostApi!.community;
      } else {
        groupDetail = await KitcodApi.instance
            .getGroupDetail(widget.feedPostApi!.communityId!);
      }
    } else if (widget.groupDetailUpdated != null) {
      groupDetail = widget.groupDetailUpdated;
    }
    setState(() {
      _initialLoading = false;
    });
  }

  @override
  void dispose() {
    super.dispose();
  }

  Future<void> deletePost() async {
    final check =
        await KitcodApi.instance.deletePostAws(widget.feedPostApi!.id!);
    if (check) {
      Navigator.popUntil(context, (final route) => route.isFirst);
      await Navigator.push(
        context,
        MaterialPageRoute(
          builder: (final context) => NewsFeed(
            headerView: HeaderView(
              backgroundColor: Colors.white.withOpacity(0.1),
              backButton: false,
              header: const Text(
                "News Feed",
                style: TextStyle(color: Colors.black),
              ),
              headerAlignment: Alignment.centerLeft,
            ),
          ),
        ),
      ).then((final value) => setState(() {}));
    }
  }

  Column reportingOptionPage(
          final BuildContext context,
          final String pageHeading,
          final List<ReportingOption> reportingOptions,
          final setModalState) =>
      Column(
        children: [
          const SizedBox(height: 5),
          Stack(
            children: [
              TextButton(
                  onPressed: () => {
                        pageController.previousPage(
                            duration: const Duration(milliseconds: 300),
                            curve: Curves.easeIn),
                        // ignore: avoid_dynamic_calls
                        setState(() => setModalState(() => subReasonIndex = 0))
                      },
                  child: const Icon(Icons.chevron_left_outlined,
                      color: Colors.black)),
              Center(
                child: Padding(
                  padding: const EdgeInsets.all(10),
                  child: Text(
                    "Reporting as $pageHeading",
                    style: const TextStyle(fontSize: 20, color: Colors.black),
                  ),
                ),
              ),
            ],
          ),
          ListView.builder(
            itemCount: reportingOptions.length,
            shrinkWrap: true,
            itemBuilder: (final context, final i) => reportOptions(
                context,
                reportingOptions[i].desc!,
                reportingOptions[i].help!,
                i,
                setModalState,
                true),
          )
        ],
      );

  Scaffold reportingReasonPage(
          final BuildContext context,
          final String pageHeading,
          final ReportingOption options,
          final setModalState) =>
      Scaffold(
        key: _modelScaffoldKey,
        body: Column(
          children: [
            const SizedBox(height: 5),
            Stack(
              children: [
                TextButton(
                    onPressed: () => {
                          pageController.previousPage(
                              duration: const Duration(milliseconds: 300),
                              curve: Curves.easeIn),
                        },
                    child: const Icon(Icons.chevron_left_outlined,
                        color: Colors.black)),
                Center(
                  child: SizedBox(
                    width: 300,
                    child: Padding(
                      padding: const EdgeInsets.all(10),
                      child: Text(
                        "Reporting as $pageHeading",
                        style:
                            const TextStyle(fontSize: 20, color: Colors.black),
                      ),
                    ),
                  ),
                ),
              ],
            ),
            Padding(
                padding: const EdgeInsets.all(20),
                child: Column(
                  children: [
                    TextFormField(
                      controller: reportTextController,
                      maxLines: 5,
                      decoration: InputDecoration(
                          border: InputBorder.none,
                          focusedBorder: OutlineInputBorder(
                            borderRadius: BorderRadius.circular(10),
                            borderSide: const BorderSide(
                              color: Colors.grey,
                            ),
                          ),
                          enabledBorder: OutlineInputBorder(
                            borderRadius: BorderRadius.circular(10),
                            borderSide: const BorderSide(
                              color: Colors.grey,
                            ),
                          ),
                          contentPadding: const EdgeInsets.only(
                              left: 15, bottom: 10, top: 10, right: 15),
                          hintText:
                              "Please add a specific reason for the report",
                          hintStyle: const TextStyle(fontSize: 18)),
                      style: const TextStyle(fontSize: 18),
                    ),
                    if (options.textRequired!)
                      Container(
                        alignment: Alignment.bottomLeft,
                        child: const Text(
                          "*Mandatory/Text Required",
                          style: TextStyle(color: Colors.red),
                          textAlign: TextAlign.left,
                        ),
                      )
                  ],
                )),
            Padding(
              padding: const EdgeInsets.only(
                  bottom: 25, left: 25, right: 25, top: 10),
              child: SizedBox(
                width: MediaQuery.of(context).size.width,
                height: 40,
                child: _isReportLoading
                    ? const Center(
                        child: Padding(
                        padding: EdgeInsets.all(8),
                        child: CircularProgressIndicator(),
                      ))
                    : TextButton(
                        onPressed: reportPost,
                        style: ButtonStyle(
                          shape:
                              MaterialStateProperty.all<RoundedRectangleBorder>(
                            RoundedRectangleBorder(
                              borderRadius: BorderRadius.circular(18),
                            ),
                          ),
                          backgroundColor:
                              MaterialStateProperty.all(Colors.indigo),
                        ),
                        child: const Text("REPORT",
                            style: TextStyle(color: Colors.white)),
                      ),
              ),
            ),
          ],
        ),
      );

  Future<void> reportPost() async {
    if (widget.reportingOptions![0].options![optionIndex]
        .subReasons![subReasonIndex].textRequired!) {
      if (reportTextController.text.isEmpty) {
        _modelScaffoldKey.currentState?.showSnackBar(const SnackBar(
          content:
              Text("Please type necessary conetent to submit your report."),
          backgroundColor: Colors.red,
        ));
        return;
      }
    }
    setState(() {
      _isReportLoading = !_isReportLoading;
    });
    final body = await KitcodApi.instance.reportPost(
        widget.reportingOptions![0].options![optionIndex],
        widget.reportingOptions![0].options![optionIndex]
            .subReasons![subReasonIndex],
        widget.feedPostApi!,
        reportTextController.text);
    if (body != null) {
      final map = jsonDecode(body) as Map<String, dynamic>;
      if (map["id"] != null) {
      } else {
        final ReportingError report =
            ReportingError.fromJson(map as Map<String, dynamic>);
        String error = "Something went wrong!";
        if (report.details != null && report.details!.resourceId != null) {
          error = report.details!.resourceId!;
        }
        _modelScaffoldKey.currentState?.showSnackBar(SnackBar(
          content: Text(error),
          backgroundColor: Colors.red,
        ));
      }
    }
    setState(() {
      _isReportLoading = false;
    });
  }

  Column reportingPage(
    final BuildContext context,
    final String pageHeading,
    final Reporting reportingOptions,
    final setModalState,
  ) =>
      Column(
        children: [
          const SizedBox(height: 5),
          Stack(
            children: [
              Center(
                child: Padding(
                  padding: const EdgeInsets.all(10),
                  child: Text(
                    pageHeading,
                    style: const TextStyle(fontSize: 20, color: Colors.black),
                  ),
                ),
              ),
            ],
          ),
          ListView.builder(
            itemCount: reportingOptions.options!.length,
            shrinkWrap: true,
            itemBuilder: (final context, final i) => reportOptions(
                context,
                reportingOptions.options![i].desc!,
                reportingOptions.options![i].help!,
                i,
                setModalState,
                false),
          )
        ],
      );

  Padding reportOptions(
          final BuildContext context,
          final String heading,
          final String description,
          final int i,
          final setModalState,
          // ignore: avoid_positional_boolean_parameters
          final bool subOption) =>
      Padding(
        padding: const EdgeInsets.only(left: 10, right: 10),
        child: TextButton(
          onPressed: () => {
            pageController.nextPage(
                duration: const Duration(milliseconds: 300),
                curve: Curves.easeIn),
            // ignore: avoid_dynamic_calls
            setState(() => setModalState(
                () => subOption ? subReasonIndex = i : optionIndex = i))
          },
          child: Row(
            children: [
              Expanded(
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Text(
                      heading,
                      style: const TextStyle(
                          fontSize: 17,
                          color: Colors.black,
                          fontWeight: FontWeight.bold),
                    ),
                    Text(
                      description,
                      style: const TextStyle(fontSize: 13, color: Colors.black),
                    ),
                  ],
                ),
              ),
              const Align(
                alignment: Alignment.bottomRight,
                child: Icon(
                  Icons.chevron_right_sharp,
                  color: Colors.black,
                ),
              )
            ],
          ),
        ),
      );

  @override
  Widget build(final BuildContext context) => _initialLoading
      ? const Align(
          alignment: Alignment.centerLeft,
          child: CircularProgressIndicator(),
        )
      : SizedBox(
          width: MediaQuery.of(context).size.width,
          child: Row(
            children: [
              if (widget.profileImageUrl != null)
                CircleAvatar(
                  radius: 23,
                  backgroundImage:
                      CachedNetworkImageProvider(widget.profileImageUrl!),
                  backgroundColor: Colors.transparent,
                )
              else
                CircleAvatar(
                  radius: 23,
                  backgroundColor: _randomColor,
                  child: Text(
                    (widget.profileDetails?.firstName?.substring(0, 1) ?? "") +
                        (widget.profileDetails?.lastName?.substring(0, 1) ??
                            ""),
                    style: const TextStyle(fontSize: 20, color: Colors.white),
                  ),
                ),
              Column(
                mainAxisAlignment: MainAxisAlignment.center,
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Padding(
                    padding: const EdgeInsets.only(left: 10),
                    child: RichText(
                      text: TextSpan(
                        text:
                            widget.name == null ? "Bilal Shahid" : widget.name,
                        style: widget.nameStyle ??
                            const TextStyle(
                                fontSize: 15,
                                fontWeight: FontWeight.bold,
                                color: Colors.black),
                        // children: <TextSpan>[
                        //   TextSpan(
                        //     text:
                        //         " ${widget.userName == null ? "bilal_shahid" : widget.userName}",
                        //     style: widget.userNameStyle ??
                        //         const TextStyle(
                        //             fontSize: 15,
                        //             color: Colors.grey,
                        //             fontWeight: FontWeight.normal),
                        //   )
                        // ]
                      ),
                    ),
                  ),
                  const SizedBox(height: 3),
                  if (groupDetail != null)
                    TextButton(
                      style: TextButton.styleFrom(
                        minimumSize: Size.zero,
                        padding: EdgeInsets.zero,
                        tapTargetSize: MaterialTapTargetSize.shrinkWrap,
                      ),
                      onPressed: () {
                        Navigator.push(
                          context,
                          MaterialPageRoute(
                              builder: (final context) => GroupPage(
                                    groupImageUrl:
                                        groupDetail!.media!.proImg == null
                                            ? "https://picsum.photos/201"
                                            : groupDetail!
                                                .media!
                                                .proImg!
                                                .resizedMediaList!
                                                .first
                                                .mediaUrl!,
                                    groupTitle: groupDetail!.name!,
                                    members:
                                        groupDetail!.rangeMembersCount == null
                                            ? ""
                                            : groupDetail!.rangeMembersCount
                                                .toString(),
                                    groupPageAbout:
                                        groupDetail!.description == null
                                            ? ""
                                            : groupDetail!.description!,
                                    groupPageStatus:
                                        groupDetail!.contentVisibilityType!,
                                    isJoined: true,
                                    profileDetailsResponse:
                                        widget.profileDetails,
                                    communityId:
                                        widget.feedPostApi!.communityId,
                                  )),
                        );
                      },
                      child: Container(
                        padding: const EdgeInsets.only(left: 10),
                        width: MediaQuery.of(context).size.width * 0.6,
                        child: Wrap(
                          children: <Widget>[
                            const Text("Posted to: ",
                                style: TextStyle(color: Colors.grey)),
                            Text(groupDetail!.name!,
                                style: widget.groupTagsStyle ??
                                    const TextStyle(color: Colors.indigo))
                          ],
                        ),
                      ),
                    ),
                  if (widget.groupDetailUpdated != null)
                    TextButton(
                      style: TextButton.styleFrom(
                        minimumSize: Size.zero,
                        padding: EdgeInsets.zero,
                        tapTargetSize: MaterialTapTargetSize.shrinkWrap,
                      ),
                      onPressed: () {},
                      child: Container(
                        padding: const EdgeInsets.only(left: 10),
                        width: MediaQuery.of(context).size.width * 0.6,
                        child: Wrap(
                          children: <Widget>[
                            const Text("Posted to: ",
                                style: TextStyle(color: Colors.grey)),
                            Text(widget.groupDetailUpdated!.name!,
                                style: widget.groupTagsStyle ??
                                    const TextStyle(color: Colors.indigo))
                          ],
                        ),
                      ),
                    ),
                  if (widget.groupDetailUpdated == null &&
                      groupDetail == null &&
                      widget.isCommentScreen == null &&
                      widget.isMemberScreen == null)
                    Container(
                      padding: const EdgeInsets.only(left: 10),
                      width: MediaQuery.of(context).size.width * 0.6,
                      child: Wrap(
                        children: <Widget>[
                          const Text("Select group to post",
                              style: TextStyle(color: Colors.grey)),
                          Text("",
                              style: widget.groupTagsStyle ??
                                  const TextStyle(color: Colors.indigo))
                        ],
                      ),
                    ),
                  if (widget.isMemberScreen != null && widget.isMemberScreen!!)
                    Container(
                      padding: const EdgeInsets.only(left: 10),
                      width: MediaQuery.of(context).size.width * 0.6,
                      child: Wrap(
                        children: <Widget>[
                          Text(role, style: widget.nameStyle),
                          Text("",
                              style: widget.groupTagsStyle ??
                                  const TextStyle(color: Colors.indigo))
                        ],
                      ),
                    )

/*
                    TextButton(
                      style: TextButton.styleFrom(
                        minimumSize: Size.zero,
                        padding: EdgeInsets.zero,
                        tapTargetSize: MaterialTapTargetSize.shrinkWrap,
                      ),
                      onPressed: () {},
                      child: ,
                    )
*/
                ],
              ),
              const Spacer(),
              if (widget.reportingOptions != null)
                IconButton(
                  alignment: Alignment.topRight,
                  icon:
                      const Icon(Icons.more_vert_rounded, color: Colors.black),
                  highlightColor: Colors.white,
                  onPressed: () => {
                    showModalBottomSheet(
                      isScrollControlled: true,
                      shape: const RoundedRectangleBorder(
                          borderRadius: BorderRadius.vertical(
                        top: Radius.circular(20),
                      )),
                      context: context,
                      builder: (final context) => SingleChildScrollView(
                        child: Column(
                          children: [
                            const SizedBox(height: 10),
                            TextButton(
                              onPressed: () => {
                                showModalBottomSheet(
                                  isScrollControlled: true,
                                  shape: const RoundedRectangleBorder(
                                      borderRadius: BorderRadius.vertical(
                                    top: Radius.circular(20),
                                  )),
                                  context: context,
                                  builder: (final context) => StatefulBuilder(
                                    builder: (final context, final setState) =>
                                        SingleChildScrollView(
                                      child: ConstrainedBox(
                                        constraints: const BoxConstraints(
                                            maxHeight: 450, minHeight: 10),
                                        child: PageView(
                                          controller: pageController,
                                          physics:
                                              const NeverScrollableScrollPhysics(),
                                          children: [
                                            if (widget.reportingOptions != null)
                                              reportingPage(
                                                  context,
                                                  "Report Post",
                                                  widget.reportingOptions![0],
                                                  setState),
                                            if (widget
                                                    .reportingOptions![0]
                                                    .options![optionIndex]
                                                    .subReasons !=
                                                null)
                                              reportingOptionPage(
                                                  context,
                                                  widget
                                                      .reportingOptions![0]
                                                      .options![optionIndex]
                                                      .type!,
                                                  widget
                                                      .reportingOptions![0]
                                                      .options![optionIndex]
                                                      .subReasons!,
                                                  setState),
                                            reportingReasonPage(
                                                context,
                                                widget
                                                            .reportingOptions![
                                                                0]
                                                            .options![
                                                                optionIndex]
                                                            .subReasons !=
                                                        null
                                                    ? widget
                                                        .reportingOptions![0]
                                                        .options![optionIndex]
                                                        .subReasons![
                                                            subReasonIndex]
                                                        .type!
                                                    : widget
                                                        .reportingOptions![0]
                                                        .options![optionIndex]
                                                        .type!,
                                                widget
                                                            .reportingOptions![
                                                                0]
                                                            .options![
                                                                optionIndex]
                                                            .subReasons !=
                                                        null
                                                    ? widget
                                                            .reportingOptions![0]
                                                            .options![optionIndex]
                                                            .subReasons![
                                                        subReasonIndex]
                                                    : widget
                                                        .reportingOptions![0]
                                                        .options![optionIndex],
                                                setState)
                                          ],
                                        ),
                                      ),
                                    ),
                                  ),
                                ),
                              },
                              child: Padding(
                                padding:
                                    const EdgeInsets.symmetric(horizontal: 10),
                                child: Row(
                                  children: const [
                                    Icon(Icons.info_outline,
                                        color: Colors.black),
                                    SizedBox(width: 5),
                                    Text(
                                      "Report Post",
                                      style: TextStyle(
                                          fontSize: 20, color: Colors.black),
                                    ),
                                  ],
                                ),
                              ),
                            ),
                            if (_userId != null && widget.feedPostApi != null)
                              if (_userId == widget.feedPostApi!.createdBy)
                                TextButton(
                                  onPressed: () => {
                                    Navigator.push(
                                      context,
                                      MaterialPageRoute(
                                        builder: (final context) =>
                                            CreateEditPost(
                                          isEdit: true,
                                          profileDetails: widget.profileDetails,
                                          feedPostApi: widget.feedPostApi,
                                          postType: const [
                                            PostType.announcement,
                                            PostType.poll
                                          ],
                                          selectedOption:
                                              widget.feedPostApi!.postType,
                                          moreOptions: const [
                                            MoreOptions.image,
                                            MoreOptions.video,
                                            MoreOptions.audio
                                          ],
                                          headerView: HeaderView(
                                            backgroundColor:
                                                Colors.white.withOpacity(0.1),
                                            leftSideIcon: IconButton(
                                              onPressed: () {
                                                Navigator.pop(context);
                                              },
                                              icon: const Icon(
                                                Icons.arrow_back_outlined,
                                                color: Colors.black,
                                              ),
                                            ),
                                            header: const Text(
                                              "Create New",
                                              style: TextStyle(
                                                  color: Colors.black),
                                            ),
                                            headerAlignment:
                                                Alignment.centerLeft,
                                          ),
                                          nameStyle: widget.nameStyle,
                                          userNameStyle: widget.userNameStyle,
                                          groupTagsStyle: widget.groupTagsStyle,
                                        ),
                                      ),
                                    ),
                                  },
                                  child: Padding(
                                    padding: const EdgeInsets.symmetric(
                                        horizontal: 10),
                                    child: Row(
                                      children: const [
                                        Icon(Icons.edit, color: Colors.black),
                                        SizedBox(width: 5),
                                        Text(
                                          "Edit Post",
                                          style: TextStyle(
                                              fontSize: 20,
                                              color: Colors.black),
                                        ),
                                      ],
                                    ),
                                  ),
                                ),
                            if (_userId != null && widget.feedPostApi != null)
                              if (_userId == widget.feedPostApi!.createdBy)
                                TextButton(
                                  onPressed: deletePost,
                                  child: Padding(
                                    padding: const EdgeInsets.symmetric(
                                        horizontal: 10),
                                    child: Row(
                                      children: const [
                                        Icon(Icons.delete, color: Colors.black),
                                        SizedBox(width: 5),
                                        Text(
                                          "Delete Post",
                                          style: TextStyle(
                                              fontSize: 20,
                                              color: Colors.black),
                                        ),
                                      ],
                                    ),
                                  ),
                                ),
                            const SizedBox(height: 10),
                          ],
                        ),
                      ),
                    ),
                  },
                ),
            ],
          ),
        );
}
