part of kitcod_uikit_flutter;

class TextBarButton extends StatelessWidget {
  const TextBarButton({
    required this.hintText,
    this.profileDetails,
    this.moreOptions,
    this.postType,
    this.nameStyle,
    this.userNameStyle,
    this.groupTagsStyle,
    this.hintTextStyle,
    this.icon,
    this.iconColor,
    this.iconBackgroundColor,
    this.headingStyle,
    this.descriptionStyle,
    final Key? key,
  }) : super(key: key);
  // -------------------------------- VARIABLES -------------------------------- //

  final UserProfile? profileDetails;
  final String hintText;
  final TextStyle? hintTextStyle;
  final IconData? icon;
  final Color? iconColor;
  final Color? iconBackgroundColor;
  final TextStyle? headingStyle;
  final TextStyle? descriptionStyle;
  final List<PostType>? postType;
  final TextStyle? nameStyle;
  final TextStyle? groupTagsStyle;
  final TextStyle? userNameStyle;
  final List<MoreOptions>? moreOptions;

  // -------------------------------- BUILD FUNCTION -------------------------------- //

  @override
  TextButton build(final BuildContext context) => TextButton(
        onPressed: () => {
          showModalBottomSheet(
            isScrollControlled: true,
            shape: const RoundedRectangleBorder(
                borderRadius: BorderRadius.vertical(
              top: Radius.circular(20),
            )),
            context: context,
            builder: (final context) => Container(
                padding: const EdgeInsets.all(8),
                child: BottomSheetDropdown(
                  profileDetails: profileDetails,
                  heading: "Select post type",
                  postType: postType ?? [PostType.announcement, PostType.poll],
                  iconColor: iconColor,
                  iconBackgroundColor: iconBackgroundColor,
                  headingStyle: headingStyle,
                  descriptionStyle: descriptionStyle,
                  nameStyle: nameStyle,
                  userNameStyle: userNameStyle,
                  groupTagsStyle: groupTagsStyle,
                  hintText: hintText,
                  icon: icon,
                  moreOptions: moreOptions,
                  hintTextStyle: hintTextStyle,
                )),
          ),
        },
        child: Padding(
          padding: const EdgeInsets.symmetric(horizontal: 5),
          child: Container(
            alignment: Alignment.centerLeft,
            height: MediaQuery.of(context).size.height * 0.06,
            width: MediaQuery.of(context).size.width,
            decoration: BoxDecoration(
                border: Border.all(
                  color: Colors.grey,
                ),
                borderRadius: const BorderRadius.all(Radius.circular(20))),
            child: Padding(
              padding: const EdgeInsets.symmetric(horizontal: 10),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  Text(hintText,
                      style:
                          hintTextStyle ?? const TextStyle(color: Colors.grey)),
                  Icon(icon ?? Icons.camera_alt_outlined,
                      color: iconColor ?? Colors.grey)
                ],
              ),
            ),
          ),
        ),
      );
}
