part of kitcod_uikit_flutter;

class NewsFeed extends StatefulWidget {
  const NewsFeed({
    required this.headerView,
    this.moreOptions,
    this.hintText,
    this.postType,
    this.nameStyle,
    this.userNameStyle,
    this.groupTagsStyle,
    this.iconColor,
    this.iconBackgroundColor,
    this.headingStyle,
    this.descriptionStyle,
    this.hintTextStyle,
    this.icon,
    this.postedTimeStyle,
    this.feedTitleStyle,
    this.feedDescriptionStyle,
    this.feedAudioTitleStyle,
    this.commentNameStyle,
    this.commentStyle,
    this.commentPostedTimeStyle,
    this.commentSubmitButtonColor,
    final Key? key,
  }) : super(key: key);

  // -------------------------------- VARIABLES -------------------------------- //

  final HeaderView headerView;
  final String? hintText;
  final TextStyle? hintTextStyle;
  final IconData? icon;
  final Color? iconColor;
  final List<PostType>? postType;
  final List<MoreOptions>? moreOptions;
  final TextStyle? nameStyle;
  final TextStyle? groupTagsStyle;
  final TextStyle? userNameStyle;
  final TextStyle? headingStyle;
  final TextStyle? descriptionStyle;
  final Color? iconBackgroundColor;
  final TextStyle? postedTimeStyle;
  final TextStyle? feedTitleStyle;
  final TextStyle? feedDescriptionStyle;
  final TextStyle? feedAudioTitleStyle;
  final TextStyle? commentNameStyle;
  final TextStyle? commentStyle;
  final TextStyle? commentPostedTimeStyle;
  final Color? commentSubmitButtonColor;

  @override
  State<NewsFeed> createState() => _NewsFeedState();
}

// -------------------------------- UI FUNCTIONS -------------------------------- //
class _NewsFeedState extends State<NewsFeed> {
  bool initialLoading = true;
  UserProfile? profileDetails;
  bool isAdminOrModerator = false;

  @override
  void initState() {
    loadInitialProfileDetailsData();
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
  }

  Future<void> loadInitialProfileDetailsData() async {
    final SharedPreferences prefs = await SharedPreferences.getInstance();
    if (prefs.getString("userProfileData") != null) {
      final String userPref = prefs.getString("userProfileData")!;
      final Map<String, dynamic> userProfileData =
          jsonDecode(userPref) as Map<String, dynamic>;
      profileDetails = UserProfile.fromJson(userProfileData);
    } else {
      profileDetails = await KitcodApi.instance.getUserProfileAWSData();
    }
    setState(() {
      for (final UserCommunit community in profileDetails!.userCommunities!) {
        if (community.userCommunityRoles != null) {
          if (community.userCommunityRoles![0].role == "ADMINISTRATOR") {
            isAdminOrModerator = true;
          }
        }
      }

      initialLoading = false;
    });
  }

  Column _feedWidget(final BuildContext context) => Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          if (isAdminOrModerator)
            TextBarButton(
              profileDetails: profileDetails,
              hintText: widget.hintText ?? "Create a Post...",
              postType:
                  widget.postType ?? [PostType.announcement, PostType.poll],
              moreOptions: widget.moreOptions,
              nameStyle: widget.nameStyle,
              userNameStyle: widget.userNameStyle,
              iconColor: widget.iconColor,
              descriptionStyle: widget.descriptionStyle,
              groupTagsStyle: widget.groupTagsStyle,
              headingStyle: widget.headingStyle,
              hintTextStyle: widget.hintTextStyle,
              iconBackgroundColor: widget.iconBackgroundColor,
            ),
          GroupsList(profileDetails: profileDetails),
          const SizedBox(height: 5),
          SizedBox(
            width: MediaQuery.of(context).size.width,
            height: 3,
            child: DecoratedBox(
              decoration: BoxDecoration(color: Colors.indigo.shade50),
            ),
          ),
          FeedPostsList(
            profileDetails: profileDetails,
            nameStyle: widget.nameStyle,
            userNameStyle: widget.userNameStyle,
            groupTagsStyle: widget.groupTagsStyle,
            descriptionStyle: widget.feedDescriptionStyle,
            feedAudioTitleStyle: widget.feedAudioTitleStyle,
            feedTitleStyle: widget.feedTitleStyle,
            postedTimeStyle: widget.postedTimeStyle,
            commentNameStyle: widget.commentNameStyle,
            commentPostedTimeStyle: widget.commentPostedTimeStyle,
            commentStyle: widget.commentStyle,
            commentSubmitButtonColor: widget.commentSubmitButtonColor,
          )
        ],
      );

  // -------------------------------- BUILD FUNCTION -------------------------------- //

  @override
  Widget build(final context) => initialLoading
      ? Scaffold(
          appBar: widget.headerView,
          body: const Center(
            child: Padding(
              padding: EdgeInsets.all(8),
              child: CircularProgressIndicator(),
            ),
          ),
        )
      : Scaffold(
          appBar: widget.headerView,
          body: _feedWidget(context),
          floatingActionButton: Visibility(
              visible: isAdminOrModerator,
              child: FloatingActionButton(
                  onPressed: () => showModalBottomSheet(
                        isScrollControlled: true,
                        shape: const RoundedRectangleBorder(
                            borderRadius: BorderRadius.vertical(
                          top: Radius.circular(20),
                        )),
                        context: context,
                        builder: (final context) => Container(
                            padding: const EdgeInsets.all(8),
                            child: BottomSheetDropdown(
                              profileDetails: profileDetails,
                              heading: "Select post type",
                              postType: widget.postType ??
                                  [PostType.announcement, PostType.poll],
                              iconColor: widget.iconColor,
                              iconBackgroundColor: widget.iconBackgroundColor,
                              headingStyle: widget.headingStyle,
                              descriptionStyle: widget.descriptionStyle,
                              nameStyle: widget.nameStyle,
                              groupTagsStyle: widget.groupTagsStyle,
                              userNameStyle: widget.userNameStyle,
                              hintText: widget.hintText ?? "Create a Post...",
                              icon: widget.icon,
                              moreOptions: widget.moreOptions,
                              hintTextStyle: widget.hintTextStyle,
                            )),
                      ),
                  child: const Icon(Icons.add_outlined))),
          floatingActionButtonLocation: FloatingActionButtonLocation.endFloat,
        );
}
