// ignore_for_file: avoid_dynamic_calls

part of kitcod_uikit_flutter;

class UserProfileView extends StatefulWidget {
  const UserProfileView({
    required this.headerView,
    required this.button,
    this.groupCardWidth,
    this.groupCardHeight,
    this.avatarSize,
    this.groupCardTitleStyle,
    this.groupCardMembersStyle,
    this.groupCardButtonColor,
    this.groupCardButtonStyle,
    this.groupPageTitleStyle,
    this.groupPageMembersStyle,
    this.groupPageStatusStyle,
    this.groupPageAboutStyle,
    this.nameStyle,
    this.profileDetailsNameStyle,
    this.profileDetailsValueStyle,
    this.postNameStyle,
    this.postUserNameStyle,
    this.postGroupTagsStyle,
    this.postFeedDescriptionStyle,
    this.postFeedAudioTitleStyle,
    this.postFeedTitleStyle,
    this.postPostedTimeStyle,
    this.commentNameStyle,
    this.commentStyle,
    this.commentPostedTimeStyle,
    this.commentSubmitButtonColor,
    this.profileDetailsResponse,
    final Key? key,
  }) : super(key: key);

  // -------------------------------- VARIABLES -------------------------------- //

  final HeaderView headerView;
  final TextStyle? nameStyle;
  final TextStyle? profileDetailsNameStyle;
  final TextStyle? profileDetailsValueStyle;
  final Button button;
  final double? groupCardWidth;
  final double? groupCardHeight;
  final double? avatarSize;
  final TextStyle? groupCardTitleStyle;
  final TextStyle? groupCardMembersStyle;
  final TextStyle? groupCardButtonStyle;
  final Color? groupCardButtonColor;
  final TextStyle? groupPageTitleStyle;
  final TextStyle? groupPageMembersStyle;
  final TextStyle? groupPageStatusStyle;
  final TextStyle? groupPageAboutStyle;
  final TextStyle? postNameStyle;
  final TextStyle? postGroupTagsStyle;
  final TextStyle? postUserNameStyle;
  final TextStyle? postFeedDescriptionStyle;
  final TextStyle? postFeedAudioTitleStyle;
  final TextStyle? postFeedTitleStyle;
  final TextStyle? postPostedTimeStyle;
  final TextStyle? commentNameStyle;
  final TextStyle? commentStyle;
  final TextStyle? commentPostedTimeStyle;
  final Color? commentSubmitButtonColor;
  final UserProfile? profileDetailsResponse;

  @override
  State<UserProfileView> createState() => _UserProfileView();
}

// -------------------------------- UI FUNCTIONS -------------------------------- //
class _UserProfileView extends State<UserProfileView> {
  late UserProfile profileDetails;
  List<Communities> userCommunitiesResponse = [];
  bool _initialLoading = true;

  List userJoinedCommunities = [];

  @override
  void initState() {
    super.initState();
    initializingAwsCalls();
  }

  @override
  void dispose() {
    super.dispose();
  }

  // ignore: avoid_void_async
  Future<void> initializingAwsCalls() async {
    profileDetails = await KitcodApi.instance
        .getOtherUserProfileID(widget.profileDetailsResponse!.id!);
    if (profileDetails.userCommunities != null) {
      for (final joinedCommunity in profileDetails.userCommunities!) {
        userJoinedCommunities
            .add({"id": joinedCommunity.communityId, "httpMethod": "Get"});
      }
      userCommunitiesResponse = await KitcodApi.instance
          .getCommunitiesBulkAWSData(userJoinedCommunities);
    }
    setState(() {
      _initialLoading = false;
    });
  }

  Padding _infoWidget(final String title, final TextStyle? titleStyle,
          final String value, final TextStyle? valueStyle) =>
      Padding(
        padding: const EdgeInsets.symmetric(horizontal: 20, vertical: 12),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          mainAxisSize: MainAxisSize.min,
          children: [
            Text(
              title,
              style: titleStyle ?? TextStyle(color: Colors.grey.shade600),
            ),
            const SizedBox(height: 4),
            Text(
              value,
              style: valueStyle ?? const TextStyle(fontWeight: FontWeight.bold),
            ),
          ],
        ),
      );

  Column _headerWidget(final BuildContext context) => Column(
        children: [
          Column(
            children: [
              Stack(clipBehavior: Clip.none, children: <Widget>[
                Container(
                  height: MediaQuery.of(context).size.height * 0.32,
                  width: MediaQuery.of(context).size.width,
                  color: Colors.indigoAccent.withOpacity(0.1),
                  child: Column(
                    children: [
                      if (profileDetails.media!.proImg != null)
                        CircleAvatar(
                          radius: MediaQuery.of(context).size.width * 0.2,
                          backgroundImage: NetworkImage(
                              // ignore: prefer_if_null_operators
                              profileDetails.media!.proImg == null
                                  ? "https://picsum.photos/201"
                                  : profileDetails.media!.proImg!
                                      .resizedMediaList!.first.mediaUrl!),
                          backgroundColor: Colors.transparent,
                        )
                      else
                        CircleAvatar(
                          radius: MediaQuery.of(context).size.width * 0.2,
                          backgroundColor: Colors.black,
                          child: Text(
                            (profileDetails.firstName?.substring(0, 1) ?? "") +
                                (profileDetails.lastName?.substring(0, 1) ??
                                    ""),
                            style: const TextStyle(
                                fontSize: 40, color: Colors.white),
                          ),
                        ),
                      Padding(
                        padding: const EdgeInsets.all(10),
                        child: Text(
                          "${profileDetails.firstName!} ${profileDetails.lastName!}",
                          style: widget.nameStyle ??
                              const TextStyle(
                                fontSize: 20,
                                fontWeight: FontWeight.bold,
                              ),
                        ),
                      ),
                    ],
                  ),
                ),
                Positioned(
                  right: 140,
                  left: 140,
                  bottom: -23,
                  child: widget.button,
                ),
              ]),
/*
              SizedBox(
                width: MediaQuery.of(context).size.width,
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    const SizedBox(height: 8),
                    //for (var profileDetails in profileDetailsArray)
                    _infoWidget(
                        "City",
                        widget.profileDetailsNameStyle,
                        profileDetails.homeLocation!.address!.city!,
                        widget.profileDetailsValueStyle),
                    _infoWidget(
                        "Email",
                        widget.profileDetailsNameStyle,
                        profileDetails.contactPoints!.first.value!,
                        widget.profileDetailsValueStyle),
                  ],
                ),
              )
*/
            ],
          ),
        ],
      );

  // SizedBox _profileDetailsWidget(final BuildContext context) => SizedBox(
  //       width: MediaQuery.of(context).size.width,
  //       child: Column(
  //         crossAxisAlignment: CrossAxisAlignment.start,
  //         children: [
  //           const SizedBox(height: 8),
  //           for (var profileDetails in profileDetailsArray)
  //             _infoWidget(profileDetails.propertyName!, profileDetailsNameStyle,
  //                 profileDetails.propertyValue!, profileDetailsValueStyle),
  //         ],
  //       ),
  //     );

  Column _groupsListWidget(final BuildContext context) => Column(
        children: [
          Padding(
            padding: const EdgeInsets.only(top: 20, right: 20, left: 30),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: const [
                Text("My groups",
                    style: TextStyle(
                        color: Colors.black, fontWeight: FontWeight.bold)),
              ],
            ),
          ),
          Padding(
            padding: const EdgeInsets.symmetric(horizontal: 10),
            child: SizedBox(
              height: 300,
              width: MediaQuery.of(context).size.width,
              child: ListView.builder(
                itemExtent: widget.groupCardHeight,
                scrollDirection: Axis.horizontal,
                shrinkWrap: true,
                itemCount: userCommunitiesResponse.length,
                itemBuilder: (final context, final i) => GroupCards(
                  communityId: userCommunitiesResponse[i].body!.id!,
                  communityCreatedBy:
                      userCommunitiesResponse[i].body!.createdBy!,
                  groupCardWidth: widget.groupCardWidth,
                  groupCardHeight: widget.groupCardHeight,
                  avatarSize: widget.avatarSize,
                  groupImageUrl:
                      userCommunitiesResponse[i].body!.media!.proImg == null
                          ? "https://picsum.photos/201"
                          : userCommunitiesResponse[i]
                              .body!
                              .media!
                              .proImg!
                              .resizedMediaList!
                              .first
                              .mediaUrl!,
                  groupTitle: userCommunitiesResponse[i].body!.name!,
                  // groupItem.groupTitle,
                  members:
                      userCommunitiesResponse[i].body!.rangeMembersCount == null
                          ? ""
                          : userCommunitiesResponse[i]
                              .body!
                              .rangeMembersCount
                              .toString(),
                  //groupItem.members,//  groupItem.membersImageUrls,
                  groupPageAbout:
                      userCommunitiesResponse[i].body!.description == null
                          ? ""
                          : userCommunitiesResponse[i].body!.description!,
                  //groupItem.groupAbout!,
                  groupPageStatus:
                      userCommunitiesResponse[i].body!.contentVisibilityType!,
                  //groupItem.groupStatus!,
                  isJoined: true,
                  // groupItem.isJoined!,
                  groupCardButtonColor: widget.groupCardButtonColor,
                  groupCardButtonStyle: widget.groupCardButtonStyle,
                  groupCardTitleStyle: widget.groupCardTitleStyle,
                  groupCardMembersStyle: widget.groupCardMembersStyle,
                  groupPageAboutStyle: widget.groupPageAboutStyle,
                  groupPageStatusStyle: widget.groupPageStatusStyle,
                  groupPageTitleStyle: widget.groupPageTitleStyle,
                  groupPageMembersStyle: widget.groupPageMembersStyle,
                  profileDetailsResponse: widget.profileDetailsResponse,
                ),
              ),
            ),
          ),
        ],
      );

  SizedBox _postsWidget(final BuildContext context) => SizedBox(
        width: MediaQuery.of(context).size.width,
        child: Column(
          children: [
            Padding(
              padding: const EdgeInsets.only(top: 10, right: 20, left: 30),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: const [
                  Text("Posts",
                      style: TextStyle(
                          color: Colors.black, fontWeight: FontWeight.bold)),
                ],
              ),
            ),
            FeedPostsList(
              profileDetails: widget.profileDetailsResponse,
              commentNameStyle: widget.commentNameStyle,
              commentPostedTimeStyle: widget.commentPostedTimeStyle,
              commentStyle: widget.commentStyle,
              commentSubmitButtonColor: widget.commentSubmitButtonColor,
              nameStyle: widget.postNameStyle,
              userNameStyle: widget.postUserNameStyle,
              groupTagsStyle: widget.postGroupTagsStyle,
              descriptionStyle: widget.postFeedDescriptionStyle,
              feedAudioTitleStyle: widget.postFeedAudioTitleStyle,
              feedTitleStyle: widget.postFeedTitleStyle,
              postedTimeStyle: widget.postPostedTimeStyle,
            ),
          ],
        ),
      );

  // -------------------------------- BUILD FUNCTION -------------------------------- //
  @override
  Widget build(final context) => _initialLoading
      ? Scaffold(
          appBar: widget.headerView,
          body: const Center(
            child: Padding(
              padding: EdgeInsets.all(8),
              child: CircularProgressIndicator(),
            ),
          ),
        )
      : Scaffold(
          appBar: widget.headerView,
          body: SingleChildScrollView(
            child: Column(children: [
              _headerWidget(context),
              //_profileDetailsWidget(context),
              _groupsListWidget(context),
              //_postsWidget(context)
            ]),
          ),
        );
}
